@@ -46,8 +46,6 @@
  *
  * It contains subnodes `label.title` and `label.subtitle` representing
  * respectively the title label and subtitle label.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -79,9 +77,9 @@ static GtkBuildableIface *parent_buildable_iface;
 
 enum {
   PROP_0,
+  PROP_SUBTITLE,
   PROP_ICON_NAME,
   PROP_ACTIVATABLE_WIDGET,
-  PROP_SUBTITLE,
   PROP_TITLE_LINES,
   PROP_SUBTITLE_LINES,
   LAST_PROP,
@@ -139,15 +137,15 @@ adw_action_row_get_property (GObject    *object,
   AdwActionRow *self = ADW_ACTION_ROW (object);
 
   switch (prop_id) {
+  case PROP_SUBTITLE:
+    g_value_set_string (value, adw_action_row_get_subtitle (self));
+    break;
   case PROP_ICON_NAME:
     g_value_set_string (value, adw_action_row_get_icon_name (self));
     break;
   case PROP_ACTIVATABLE_WIDGET:
     g_value_set_object (value, (GObject *) adw_action_row_get_activatable_widget (self));
     break;
-  case PROP_SUBTITLE:
-    g_value_set_string (value, adw_action_row_get_subtitle (self));
-    break;
   case PROP_SUBTITLE_LINES:
     g_value_set_int (value, adw_action_row_get_subtitle_lines (self));
     break;
@@ -168,15 +166,15 @@ adw_action_row_set_property (GObject      *object,
   AdwActionRow *self = ADW_ACTION_ROW (object);
 
   switch (prop_id) {
+  case PROP_SUBTITLE:
+    adw_action_row_set_subtitle (self, g_value_get_string (value));
+    break;
   case PROP_ICON_NAME:
     adw_action_row_set_icon_name (self, g_value_get_string (value));
     break;
   case PROP_ACTIVATABLE_WIDGET:
     adw_action_row_set_activatable_widget (self, (GtkWidget*) g_value_get_object (value));
     break;
-  case PROP_SUBTITLE:
-    adw_action_row_set_subtitle (self, g_value_get_string (value));
-    break;
   case PROP_SUBTITLE_LINES:
     adw_action_row_set_subtitle_lines (self, g_value_get_int (value));
     break;
@@ -227,12 +225,25 @@ adw_action_row_class_init (AdwActionRowClass *klass)
 
   klass->activate = adw_action_row_activate_real;
 
+  /**
+   * AdwActionRow:subtitle: (attributes org.gtk.Property.get=adw_action_row_get_subtitle org.gtk.Property.set=adw_action_row_set_subtitle)
+   *
+   * The subtitle for this row.
+   *
+   * The subtitle is interpreted as Pango markup unless
+   * [property@PreferencesRow:use-markup] is set to `FALSE`.
+   */
+  props[PROP_SUBTITLE] =
+    g_param_spec_string ("subtitle", NULL, NULL,
+                         "",
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwActionRow:icon-name: (attributes org.gtk.Property.get=adw_action_row_get_icon_name org.gtk.Property.set=adw_action_row_set_icon_name)
    *
    * The icon name for this row.
    *
-   * Since: 1.0
+   * Deprecated: 1.3: Use [method@ActionRow.add_prefix] to add an icon.
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -251,34 +262,18 @@ adw_action_row_class_init (AdwActionRowClass *klass)
    *
    * The target widget will be activated by emitting the
    * [signal@Gtk.Widget::mnemonic-activate] signal on it.
-   *
-   * Since: 1.0
    */
   props[PROP_ACTIVATABLE_WIDGET] =
     g_param_spec_object ("activatable-widget", NULL, NULL,
                          GTK_TYPE_WIDGET,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwActionRow:subtitle: (attributes org.gtk.Property.get=adw_action_row_get_subtitle org.gtk.Property.set=adw_action_row_set_subtitle)
-   *
-   * The subtitle for this row.
-   *
-   * Since: 1.0
-   */
-  props[PROP_SUBTITLE] =
-    g_param_spec_string ("subtitle", NULL, NULL,
-                         "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   /**
    * AdwActionRow:title-lines: (attributes org.gtk.Property.get=adw_action_row_get_title_lines org.gtk.Property.set=adw_action_row_set_title_lines)
    *
    * The number of lines at the end of which the title label will be ellipsized.
    *
    * If the value is 0, the number of lines won't be limited.
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE_LINES] =
     g_param_spec_int ("title-lines", NULL, NULL,
@@ -293,8 +288,6 @@ adw_action_row_class_init (AdwActionRowClass *klass)
    * ellipsized.
    *
    * If the value is 0, the number of lines won't be limited.
-   *
-   * Since: 1.0
    */
   props[PROP_SUBTITLE_LINES] =
     g_param_spec_int ("subtitle-lines", NULL, NULL,
@@ -308,8 +301,6 @@ adw_action_row_class_init (AdwActionRowClass *klass)
    * AdwActionRow::activated:
    *
    * This signal is emitted after the row has been activated.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_ACTIVATED] =
     g_signal_new ("activated",
@@ -338,7 +329,6 @@ adw_action_row_init (AdwActionRow *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   g_signal_connect (self, "notify::parent", G_CALLBACK (parent_cb), NULL);
-
 }
 
 static void
@@ -375,8 +365,6 @@ adw_action_row_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwActionRow`.
  *
  * Returns: the newly created `AdwActionRow`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_action_row_new (void)
@@ -384,6 +372,80 @@ adw_action_row_new (void)
   return g_object_new (ADW_TYPE_ACTION_ROW, NULL);
 }
 
+/**
+ * adw_action_row_add_prefix:
+ * @self: an action row
+ * @widget: a widget
+ *
+ * Adds a prefix widget to @self.
+ */
+void
+adw_action_row_add_prefix (AdwActionRow *self,
+                           GtkWidget    *widget)
+{
+  AdwActionRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  priv = adw_action_row_get_instance_private (self);
+
+  gtk_box_prepend (priv->prefixes, widget);
+  gtk_widget_set_visible (GTK_WIDGET (priv->prefixes), TRUE);
+}
+
+/**
+ * adw_action_row_add_suffix:
+ * @self: an action row
+ * @widget: a widget
+ *
+ * Adds a suffix widget to @self.
+ */
+void
+adw_action_row_add_suffix (AdwActionRow *self,
+                           GtkWidget    *widget)
+{
+  AdwActionRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  priv = adw_action_row_get_instance_private (self);
+
+  gtk_box_append (priv->suffixes, widget);
+  gtk_widget_set_visible (GTK_WIDGET (priv->suffixes), TRUE);
+}
+
+/**
+ * adw_action_row_remove:
+ * @self: an action row
+ * @widget: the child to be removed
+ *
+ * Removes a child from @self.
+ */
+void
+adw_action_row_remove (AdwActionRow *self,
+                       GtkWidget    *child)
+{
+  AdwActionRowPrivate *priv;
+  GtkWidget *parent;
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+
+  priv = adw_action_row_get_instance_private (self);
+
+  parent = gtk_widget_get_parent (child);
+
+  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->suffixes)) {
+    gtk_box_remove (GTK_BOX (parent), child);
+    gtk_widget_set_visible (parent, gtk_widget_get_first_child (parent) != NULL);
+  }
+  else {
+    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  }
+}
+
 /**
  * adw_action_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
  * @self: an action row
@@ -391,8 +453,6 @@ adw_action_row_new (void)
  * Gets the subtitle for @self.
  *
  * Returns: (nullable): the subtitle for @self
- *
- * Since: 1.0
  */
 const char *
 adw_action_row_get_subtitle (AdwActionRow *self)
@@ -415,8 +475,6 @@ adw_action_row_get_subtitle (AdwActionRow *self)
  *
  * The subtitle is interpreted as Pango markup unless
  * [property@PreferencesRow:use-markup] is set to `FALSE`.
- *
- * Since: 1.0
  */
 void
 adw_action_row_set_subtitle (AdwActionRow *self,
@@ -444,7 +502,7 @@ adw_action_row_set_subtitle (AdwActionRow *self,
  *
  * Returns: (nullable): the icon name for @self
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ActionRow.add_prefix] to add an icon.
  */
 const char *
 adw_action_row_get_icon_name (AdwActionRow *self)
@@ -465,7 +523,7 @@ adw_action_row_get_icon_name (AdwActionRow *self)
  *
  * Sets the icon name for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ActionRow.add_prefix] to add an icon.
  */
 void
 adw_action_row_set_icon_name (AdwActionRow *self,
@@ -494,8 +552,6 @@ adw_action_row_set_icon_name (AdwActionRow *self,
  * Gets the widget activated when @self is activated.
  *
  * Returns: (nullable) (transfer none): the activatable widget for @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_action_row_get_activatable_widget (AdwActionRow *self)
@@ -529,7 +585,12 @@ activatable_widget_weak_notify (gpointer  data,
  *
  * Sets the widget to activate when @self is activated.
  *
- * Since: 1.0
+ * The row can be activated either by clicking on it, calling
+ * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+ * See the [property@PreferencesRow:use-underline] property to enable mnemonics.
+ *
+ * The target widget will be activated by emitting the
+ * [signal@Gtk.Widget::mnemonic-activate] signal on it.
  */
 void
 adw_action_row_set_activatable_widget (AdwActionRow *self,
@@ -586,12 +647,8 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
  * Gets the number of lines at the end of which the title label will be
  * ellipsized.
  *
- * If the value is 0, the number of lines won't be limited.
- *
  * Returns: the number of lines at the end of which the title label will be
  *   ellipsized
- *
- * Since: 1.0
  */
 int
 adw_action_row_get_title_lines (AdwActionRow *self)
@@ -614,8 +671,6 @@ adw_action_row_get_title_lines (AdwActionRow *self)
  * ellipsized.
  *
  * If the value is 0, the number of lines won't be limited.
- *
- * Since: 1.0
  */
 void
 adw_action_row_set_title_lines (AdwActionRow *self,
@@ -646,12 +701,8 @@ adw_action_row_set_title_lines (AdwActionRow *self,
  * Gets the number of lines at the end of which the subtitle label will be
  * ellipsized.
  *
- * If the value is 0, the number of lines won't be limited.
- *
  * Returns: the number of lines at the end of which the subtitle label will be
  *   ellipsized
- *
- * Since: 1.0
  */
 int
 adw_action_row_get_subtitle_lines (AdwActionRow *self)
@@ -674,8 +725,6 @@ adw_action_row_get_subtitle_lines (AdwActionRow *self)
  * ellipsized.
  *
  * If the value is 0, the number of lines won't be limited.
- *
- * Since: 1.0
  */
 void
 adw_action_row_set_subtitle_lines (AdwActionRow *self,
@@ -699,92 +748,11 @@ adw_action_row_set_subtitle_lines (AdwActionRow *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SUBTITLE_LINES]);
 }
 
-/**
- * adw_action_row_add_prefix:
- * @self: an action row
- * @widget: a widget
- *
- * Adds a prefix widget to @self.
- *
- * Since: 1.0
- */
-void
-adw_action_row_add_prefix (AdwActionRow *self,
-                           GtkWidget    *widget)
-{
-  AdwActionRowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_ACTION_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  priv = adw_action_row_get_instance_private (self);
-
-  gtk_box_prepend (priv->prefixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->prefixes));
-}
-
-/**
- * adw_action_row_add_suffix:
- * @self: an action row
- * @widget: a widget
- *
- * Adds a suffix widget to @self.
- *
- * Since: 1.0
- */
-void
-adw_action_row_add_suffix (AdwActionRow *self,
-                           GtkWidget    *widget)
-{
-  AdwActionRowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_ACTION_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  priv = adw_action_row_get_instance_private (self);
-
-  gtk_box_append (priv->suffixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->suffixes));
-}
-
-/**
- * adw_action_row_remove:
- * @self: an action row
- * @widget: the child to be removed
- *
- * Removes a child from @self.
- *
- * Since: 1.0
- */
-void
-adw_action_row_remove (AdwActionRow *self,
-                       GtkWidget    *child)
-{
-  AdwActionRowPrivate *priv;
-  GtkWidget *parent;
-
-  g_return_if_fail (ADW_IS_ACTION_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (child));
-
-  priv = adw_action_row_get_instance_private (self);
-
-  parent = gtk_widget_get_parent (child);
-
-  if (parent == GTK_WIDGET (priv->prefixes))
-    gtk_box_remove (priv->prefixes, child);
-  else if (parent == GTK_WIDGET (priv->suffixes))
-    gtk_box_remove (priv->suffixes, child);
-  else
-    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
-}
-
 /**
  * adw_action_row_activate:
  * @self: an action row
  *
  * Activates @self.
- *
- * Since: 1.0
  */
 void
 adw_action_row_activate (AdwActionRow *self)
