@@ -15,7 +15,6 @@
 
 #include "adw-avatar.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 
 #define NUMBER_OF_COLORS 14
 
@@ -45,8 +44,6 @@
  * ## CSS nodes
  *
  * `AdwAvatar` has a single CSS node with name `avatar`.
- *
- * Since: 1.0
  */
 
 struct _AdwAvatar
@@ -213,6 +210,24 @@ update_font_size (AdwAvatar *self)
   pango_attr_list_unref (attributes);
 }
 
+static void
+update_custom_image_snapshot (AdwAvatar *self)
+{
+  GtkSnapshot *snapshot = gtk_snapshot_new ();
+  GdkPaintable *square_image;
+  int height = gdk_paintable_get_intrinsic_height (self->custom_image_source);
+  int width = gdk_paintable_get_intrinsic_width (self->custom_image_source);
+  int size = MIN (width, height);
+
+  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - width) / 2.f, (size - height) / 2.f));
+  gdk_paintable_snapshot (self->custom_image_source, snapshot, width, height);
+
+  square_image = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
+  gtk_image_set_from_paintable (self->custom_image, square_image);
+
+  g_object_unref (square_image);
+}
+
 static void
 adw_avatar_get_property (GObject    *object,
                          guint       property_id,
@@ -326,8 +341,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * The name of an icon to use as a fallback.
    *
    * If no name is set, `avatar-default-symbolic` will be used.
-   *
-   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -341,8 +354,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    *
    * It's only used to generate the color if [property@Avatar:show-initials] is
    * `FALSE`.
-   *
-   * Since: 1.0
    */
   props[PROP_TEXT] =
     g_param_spec_string ("text", NULL, NULL,
@@ -355,8 +366,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Whether initials are used instead of an icon on the fallback avatar.
    *
    * See [property@Avatar:icon-name] for how to change the fallback icon.
-   *
-   * Since: 1.0
    */
   props[PROP_SHOW_INITIALS] =
     g_param_spec_boolean ("show-initials", NULL, NULL,
@@ -366,9 +375,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
   /**
    * AdwAvatar:custom-image: (attributes org.gtk.Property.get=adw_avatar_get_custom_image org.gtk.Property.set=adw_avatar_set_custom_image)
    *
-   * A custom image to use instead of initials or icon.
+   * A custom image paintable.
    *
-   * Since: 1.0
+   * Custom image is displayed instead of initials or icon.
    */
   props[PROP_CUSTOM_IMAGE] =
     g_param_spec_object ("custom-image", NULL, NULL,
@@ -379,8 +388,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * AdwAvatar:size: (attributes org.gtk.Property.get=adw_avatar_get_size org.gtk.Property.set=adw_avatar_set_size)
    *
    * The size of the avatar.
-   *
-   * Since: 1.0
    */
   props[PROP_SIZE] =
     g_param_spec_int ("size", NULL, NULL,
@@ -431,8 +438,6 @@ adw_avatar_init (AdwAvatar *self)
  * Creates a new `AdwAvatar`.
  *
  * Returns: the newly created `AdwAvatar`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_avatar_new (int         size,
@@ -453,8 +458,6 @@ adw_avatar_new (int         size,
  * Gets the name of an icon to use as a fallback.
  *
  * Returns: (nullable): the icon name
- *
- * Since: 1.0
  */
 const char *
 adw_avatar_get_icon_name (AdwAvatar *self)
@@ -472,8 +475,6 @@ adw_avatar_get_icon_name (AdwAvatar *self)
  * Sets the name of an icon to use as a fallback.
  *
  * If no name is set, `avatar-default-symbolic` will be used.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_icon_name (AdwAvatar  *self,
@@ -500,8 +501,6 @@ adw_avatar_set_icon_name (AdwAvatar  *self,
  *
  * Returns: (nullable): the text used to generate the fallback initials and
  *   color
- *
- * Since: 1.0
  */
 const char *
 adw_avatar_get_text (AdwAvatar *self)
@@ -518,7 +517,8 @@ adw_avatar_get_text (AdwAvatar *self)
  *
  * Sets the text used to generate the fallback initials and color.
  *
- * Since: 1.0
+ * It's only used to generate the color if [property@Avatar:show-initials] is
+ * `FALSE`.
  */
 void
 adw_avatar_set_text (AdwAvatar  *self,
@@ -548,8 +548,6 @@ adw_avatar_set_text (AdwAvatar  *self,
  * Gets whether initials are used instead of an icon on the fallback avatar.
  *
  * Returns: whether initials are used instead of an icon as fallback
- *
- * Since: 1.0
  */
 gboolean
 adw_avatar_get_show_initials (AdwAvatar *self)
@@ -566,7 +564,7 @@ adw_avatar_get_show_initials (AdwAvatar *self)
  *
  * Sets whether to use initials instead of an icon on the fallback avatar.
  *
- * Since: 1.0
+ * See [property@Avatar:icon-name] for how to change the fallback icon.
  */
 void
 adw_avatar_set_show_initials (AdwAvatar *self,
@@ -593,8 +591,6 @@ adw_avatar_set_show_initials (AdwAvatar *self,
  * Gets the custom image paintable.
  *
  * Returns: (nullable) (transfer none): the custom image
- *
- * Since: 1.0
  */
 GdkPaintable *
 adw_avatar_get_custom_image (AdwAvatar *self)
@@ -611,7 +607,7 @@ adw_avatar_get_custom_image (AdwAvatar *self)
  *
  * Sets the custom image paintable.
  *
- * Since: 1.0
+ * Custom image is displayed instead of initials or icon.
  */
 void
 adw_avatar_set_custom_image (AdwAvatar    *self,
@@ -623,6 +619,11 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
   if (self->custom_image_source == custom_image)
     return;
 
+  if (self->custom_image_source) {
+    g_signal_handlers_disconnect_by_func (self->custom_image_source,
+                                          update_custom_image_snapshot, self);
+  }
+
   g_set_object (&self->custom_image_source, custom_image);
 
   if (custom_image) {
@@ -632,17 +633,10 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
     if (height == width) {
       gtk_image_set_from_paintable (self->custom_image, custom_image);
     } else {
-      GtkSnapshot *snapshot = gtk_snapshot_new ();
-      GdkPaintable *square_image;
-      int size = MIN (width, height);
-
-      gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - width) / 2.f, (size - height) / 2.f));
-      gdk_paintable_snapshot (custom_image, snapshot, width, height);
-
-      square_image = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
-      gtk_image_set_from_paintable (self->custom_image, square_image);
-
-      g_object_unref (square_image);
+      update_custom_image_snapshot (self);
+      g_signal_connect_swapped (custom_image, "invalidate-contents",
+                                G_CALLBACK (update_custom_image_snapshot),
+                                self);
     }
 
     gtk_widget_add_css_class (self->gizmo, "image");
@@ -664,8 +658,6 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
  * Gets the size of the avatar.
  *
  * Returns: the size of the avatar
- *
- * Since: 1.0
  */
 int
 adw_avatar_get_size (AdwAvatar *self)
@@ -681,8 +673,6 @@ adw_avatar_get_size (AdwAvatar *self)
  * @size: The size of the avatar
  *
  * Sets the size of the avatar.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_size (AdwAvatar *self,
@@ -720,8 +710,6 @@ adw_avatar_set_size (AdwAvatar *self,
  * This can be used to export the fallback avatar.
  *
  * Returns: (transfer full): the texture
- *
- * Since: 1.0
  */
 GdkTexture *
 adw_avatar_draw_to_texture (AdwAvatar *self,
@@ -733,14 +721,20 @@ adw_avatar_draw_to_texture (AdwAvatar *self,
   GtkNative *native;
   GskRenderer *renderer;
   int size;
+  graphene_matrix_t transform;
 
   g_return_val_if_fail (ADW_IS_AVATAR (self), NULL);
   g_return_val_if_fail (scale_factor > 0, NULL);
 
   size = self->size * scale_factor;
 
+  g_assert (gtk_widget_compute_transform (GTK_WIDGET (self),
+                                          self->gizmo,
+                                          &transform));
+
   snapshot = gtk_snapshot_new ();
   gtk_snapshot_scale (snapshot, scale_factor, scale_factor);
+  gtk_snapshot_transform_matrix (snapshot, &transform);
   GTK_WIDGET_GET_CLASS (self)->snapshot (GTK_WIDGET (self), snapshot);
 
   node = gtk_snapshot_free_to_node (snapshot);
