@@ -80,6 +80,7 @@ typedef struct
   gboolean show_apply_button;
   gboolean text_changed;
   gboolean show_indicator;
+  gboolean activates_default;
 } AdwEntryRowPrivate;
 
 static void adw_entry_row_editable_init (GtkEditableInterface *iface);
@@ -97,7 +98,9 @@ enum {
   PROP_SHOW_APPLY_BUTTON,
   PROP_INPUT_HINTS,
   PROP_INPUT_PURPOSE,
+  PROP_ATTRIBUTES,
   PROP_ENABLE_EMOJI_COMPLETION,
+  PROP_ACTIVATES_DEFAULT,
   PROP_LAST_PROP,
 };
 
@@ -105,6 +108,7 @@ static GParamSpec *props[PROP_LAST_PROP];
 
 enum {
   SIGNAL_APPLY,
+  SIGNAL_ENTRY_ACTIVATED,
   SIGNAL_LAST_SIGNAL,
 };
 
@@ -217,7 +221,7 @@ pressed_cb (GtkGesture  *gesture,
     return;
   }
 
-  gtk_widget_grab_focus (GTK_WIDGET (priv->text));
+  gtk_text_grab_focus_without_selecting (GTK_TEXT (priv->text));
 
   gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_CLAIMED);
 }
@@ -241,8 +245,14 @@ text_activated_cb (AdwEntryRow *self)
 {
   AdwEntryRowPrivate *priv = adw_entry_row_get_instance_private (self);
 
-  if (gtk_widget_get_visible (priv->apply_button))
+  if (gtk_widget_get_visible (priv->apply_button)) {
     apply_button_clicked_cb (self);
+  } else {
+    if (priv->activates_default)
+      gtk_widget_activate_default (GTK_WIDGET (self));
+
+    g_signal_emit (self, signals[SIGNAL_ENTRY_ACTIVATED], 0);
+  }
 }
 
 static void
@@ -356,9 +366,15 @@ adw_entry_row_get_property (GObject     *object,
   case PROP_INPUT_PURPOSE:
     g_value_set_enum (value, adw_entry_row_get_input_purpose (self));
     break;
+  case PROP_ATTRIBUTES:
+    g_value_set_boxed (value, adw_entry_row_get_attributes (self));
+    break;
   case PROP_ENABLE_EMOJI_COMPLETION:
     g_value_set_boolean (value, adw_entry_row_get_enable_emoji_completion (self));
     break;
+  case PROP_ACTIVATES_DEFAULT:
+    g_value_set_boolean (value, adw_entry_row_get_activates_default (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -393,9 +409,15 @@ adw_entry_row_set_property (GObject       *object,
   case PROP_INPUT_PURPOSE:
     adw_entry_row_set_input_purpose (self, g_value_get_enum (value));
     break;
+  case PROP_ATTRIBUTES:
+    adw_entry_row_set_attributes (self, g_value_get_boxed (value));
+    break;
   case PROP_ENABLE_EMOJI_COMPLETION:
     adw_entry_row_set_enable_emoji_completion (self, g_value_get_boolean (value));
     break;
+  case PROP_ACTIVATES_DEFAULT:
+    adw_entry_row_set_activates_default (self, g_value_get_boolean (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -460,10 +482,10 @@ adw_entry_row_class_init (AdwEntryRowClass *klass)
    * Since: 1.2
    */
   props[PROP_INPUT_HINTS] =
-      g_param_spec_flags ("input-hints", NULL, NULL,
-                          GTK_TYPE_INPUT_HINTS,
-                          GTK_INPUT_HINT_NONE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_flags ("input-hints", NULL, NULL,
+                        GTK_TYPE_INPUT_HINTS,
+                        GTK_INPUT_HINT_NONE,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwEntryRow:input-purpose: (attributes org.gtk.Property.get=adw_entry_row_get_input_purpose org.gtk.Property.set=adw_entry_row_set_input_purpose)
@@ -475,25 +497,51 @@ adw_entry_row_class_init (AdwEntryRowClass *klass)
    * Since: 1.2
    */
   props[PROP_INPUT_PURPOSE] =
-      g_param_spec_enum ("input-purpose", NULL, NULL,
-                         GTK_TYPE_INPUT_PURPOSE,
-                         GTK_INPUT_PURPOSE_FREE_FORM,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_enum ("input-purpose", NULL, NULL,
+                       GTK_TYPE_INPUT_PURPOSE,
+                       GTK_INPUT_PURPOSE_FREE_FORM,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwEntryRow:attributes: (attributes org.gtk.Property.get=adw_entry_row_get_attributes org.gtk.Property.set=adw_entry_row_set_attributes)
+   *
+   * A list of Pango attributes to apply to the text of the embedded entry.
+   *
+   * The [struct@Pango.Attribute]'s `start_index` and `end_index` must refer to
+   * the [class@Gtk.EntryBuffer] text, i.e. without the preedit string.
+   *
+   * Since: 1.2
+   */
+  props[PROP_ATTRIBUTES] =
+    g_param_spec_boxed ("attributes", NULL, NULL,
+                        PANGO_TYPE_ATTR_LIST,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwEntryRow:enable-emoji-completion: (attributes org.gtk.Property.get=adw_entry_row_get_enable_emoji_completion org.gtk.Property.set=adw_entry_row_set_enable_emoji_completion)
    *
    * Whether to suggest emoji replacements on the entry row.
    *
-   * Emoji replacement is done with :-delimited names,
-   * like `:heart:`.
+   * Emoji replacement is done with :-delimited names, like `:heart:`.
    *
    * Since: 1.2
    */
   props[PROP_ENABLE_EMOJI_COMPLETION] =
-      g_param_spec_boolean ("enable-emoji-completion", NULL, NULL,
-                            FALSE,
-                            G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_boolean ("enable-emoji-completion", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwEntryRow:activates-default: (attributes org.gtk.Property.get=adw_entry_row_get_activates_default org.gtk.Property.set=adw_entry_row_set_activates_default)
+   *
+   * Whether activating the embedded entry can activate the default widget.
+   *
+   * Since: 1.2
+   */
+  props[PROP_ACTIVATES_DEFAULT] =
+    g_param_spec_boolean ("activates-default", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
@@ -517,6 +565,22 @@ adw_entry_row_class_init (AdwEntryRowClass *klass)
                   G_TYPE_NONE,
                   0);
 
+  /**
+   * AdwEntryRow::entry-activated:
+   *
+   * Emitted when the embedded entry is activated.
+   *
+   * Since: 1.2
+   */
+  signals[SIGNAL_ENTRY_ACTIVATED] =
+    g_signal_new ("entry-activated",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  0);
+
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-entry-row.ui");
   gtk_widget_class_bind_template_child_private (widget_class, AdwEntryRow, header);
@@ -649,7 +713,7 @@ adw_entry_row_add_prefix (AdwEntryRow *self,
   priv = adw_entry_row_get_instance_private (self);
 
   gtk_box_prepend (priv->prefixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->prefixes));
+  gtk_widget_set_visible (GTK_WIDGET (priv->prefixes), TRUE);
 }
 
 /**
@@ -673,7 +737,7 @@ adw_entry_row_add_suffix (AdwEntryRow *self,
   priv = adw_entry_row_get_instance_private (self);
 
   gtk_box_append (priv->suffixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->suffixes));
+  gtk_widget_set_visible (GTK_WIDGET (priv->suffixes), TRUE);
 }
 
 /**
@@ -699,12 +763,13 @@ adw_entry_row_remove (AdwEntryRow *self,
 
   parent = gtk_widget_get_parent (child);
 
-  if (parent == GTK_WIDGET (priv->prefixes))
-    gtk_box_remove (priv->prefixes, child);
-  else if (parent == GTK_WIDGET (priv->suffixes))
-    gtk_box_remove (priv->suffixes, child);
-  else
+  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->suffixes)) {
+    gtk_box_remove (GTK_BOX (parent), child);
+    gtk_widget_set_visible (parent, gtk_widget_get_first_child (parent) != NULL);
+  }
+  else {
     ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  }
 }
 
 /**
@@ -736,6 +801,14 @@ adw_entry_row_get_show_apply_button (AdwEntryRow *self)
  *
  * Sets whether @self can show the apply button.
  *
+ * When set to `TRUE`, typing text in the entry will reveal an apply button.
+ * Clicking it or pressing the <kbd>Enter</kbd> key will hide the button and
+ * emit the [signal@EntryRow::apply] signal.
+ *
+ * This is useful if changing the entry contents can trigger an expensive
+ * operation, e.g. network activity, to avoid triggering it after typing every
+ * character.
+ *
  * Since: 1.2
  */
 void
@@ -808,6 +881,9 @@ adw_entry_row_set_input_hints (AdwEntryRow   *self,
 
   priv = adw_entry_row_get_instance_private (self);
 
+  if (hints == adw_entry_row_get_input_hints (self))
+    return;
+
   gtk_text_set_input_hints (GTK_TEXT (priv->text), hints);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_INPUT_HINTS]);
@@ -856,20 +932,72 @@ adw_entry_row_set_input_purpose (AdwEntryRow     *self,
 
   priv = adw_entry_row_get_instance_private (self);
 
+  if (purpose == adw_entry_row_get_input_purpose (self))
+    return;
+
   gtk_text_set_input_purpose (GTK_TEXT (priv->text), purpose);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_INPUT_PURPOSE]);
 }
 
+/**
+ * adw_entry_row_get_attributes: (attributes org.gtk.Method.get_property=attributes)
+ * @self: an entry row
+ *
+ * Gets Pango attributes applied to the text of the embedded entry.
+ *
+ * Returns: (nullable): the list of attributes
+ *
+ * Since: 1.2
+ */
+PangoAttrList *
+adw_entry_row_get_attributes (AdwEntryRow *self)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_ENTRY_ROW (self), NULL);
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  return gtk_text_get_attributes (GTK_TEXT (priv->text));
+}
+
+/**
+ * adw_entry_row_set_attributes: (attributes org.gtk.Method.set_property=attributes)
+ * @self: an entry row
+ * @attributes: (nullable): a list of attributes
+ *
+ * Sets Pango attributes to apply to the text of the embedded entry.
+ *
+ * The [struct@Pango.Attribute]'s `start_index` and `end_index` must refer to
+ * the [class@Gtk.EntryBuffer] text, i.e. without the preedit string.
+ *
+ * Since: 1.2
+ */
+void
+adw_entry_row_set_attributes (AdwEntryRow   *self,
+                              PangoAttrList *attributes)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_ENTRY_ROW (self));
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  if (attributes == adw_entry_row_get_attributes (self))
+    return;
+
+  gtk_text_set_attributes (GTK_TEXT (priv->text), attributes);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ATTRIBUTES]);
+}
+
 /**
  * adw_entry_row_get_enable_emoji_completion: (attributes org.gtk.Method.get_property=enable-emoji-completion)
  * @self: an entry row
  *
  * Gets whether to suggest emoji replacements on @self.
  *
- * Emoji replacement is done with :-delimited names,
- * like `:heart:`.
- *
  * Returns: whether or not emoji completion is enabled
  *
  * Since: 1.2
@@ -893,8 +1021,7 @@ adw_entry_row_get_enable_emoji_completion (AdwEntryRow *self)
  *
  * Sets whether to suggest emoji replacements on @self.
  *
- * Emoji replacement is done with :-delimited names,
- * like `:heart:`.
+ * Emoji replacement is done with :-delimited names, like `:heart:`.
  *
  * Since: 1.2
  */
@@ -908,11 +1035,63 @@ adw_entry_row_set_enable_emoji_completion (AdwEntryRow *self,
 
   priv = adw_entry_row_get_instance_private (self);
 
+  enable_emoji_completion = !!enable_emoji_completion;
+
+  if (enable_emoji_completion == adw_entry_row_get_enable_emoji_completion (self))
+    return;
+
   gtk_text_set_enable_emoji_completion (GTK_TEXT (priv->text), enable_emoji_completion);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ENABLE_EMOJI_COMPLETION]);
 }
 
+/**
+ * adw_entry_row_get_activates_default: (attributes org.gtk.Method.get_property=activates-default)
+ * @self: an entry row
+ *
+ * Gets whether activating the embedded entry can activate the default widget.
+ *
+ * Returns: whether to activate the default widget
+ */
+gboolean
+adw_entry_row_get_activates_default (AdwEntryRow *self)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_ENTRY_ROW (self), FALSE);
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  return priv->activates_default;
+}
+
+/**
+ * adw_entry_row_set_activates_default: (attributes org.gtk.Method.set_property=activates-default)
+ * @self: an entry row
+ * @activates: whether to activate the default widget
+ *
+ * Sets whether activating the embedded entry can activate the default widget.
+ *
+ * Since: 1.2
+ */
+void
+adw_entry_row_set_activates_default (AdwEntryRow *self,
+                                     gboolean     activates)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_ENTRY_ROW (self));
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  if (priv->activates_default == activates)
+    return;
+
+  priv->activates_default = activates;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACTIVATES_DEFAULT]);
+}
+
 void
 adw_entry_row_set_indicator_icon_name (AdwEntryRow *self,
                                        const char  *icon_name)
@@ -955,3 +1134,27 @@ adw_entry_row_set_show_indicator (AdwEntryRow *self,
 
   update_empty (self);
 }
+
+/**
+ * adw_entry_grab_focus_without_selecting:
+ * @self: an entry row
+ *
+ * Causes @self to have keyboard focus without selecting the text.
+ * 
+ * See [method@Gtk.Text.grab_focus_without_selecting] for more information.
+ *
+ * Returns: whether the focus is now inside @self
+ *
+ * Since: 1.3
+ */
+gboolean
+adw_entry_row_grab_focus_without_selecting (AdwEntryRow *self)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_ENTRY_ROW (self), FALSE);
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  return gtk_text_grab_focus_without_selecting (GTK_TEXT (priv->text));
+}
