@@ -12,7 +12,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 #include "adw-shadow-helper-private.h"
 #include "adw-spring-animation.h"
 #include "adw-swipeable.h"
@@ -73,15 +72,13 @@
  *
  * The `AdwFlap` implementation of the [iface@Gtk.Buildable] interface supports
  * setting the flap child by specifying “flap” as the “type” attribute of a
- * <child> element, and separator by specifying “separator”. Specifying
+ * `<child>` element, and separator by specifying “separator”. Specifying
  * “content” child type or omitting it results in setting the content child.
  *
  * ## CSS nodes
  *
  * `AdwFlap` has a single CSS node with name `flap`. The node will get the style
  * classes `.folded` when it is folded, and `.unfolded` when it's not.
- *
- * Since: 1.0
  */
 
 /**
@@ -93,8 +90,6 @@
  *   space.
  *
  * Describes the possible folding behavior of a [class@Flap] widget.
- *
- * Since: 1.0
  */
 
 /**
@@ -114,8 +109,6 @@
  * [property@Flap:swipe-to-open] and [property@Flap:swipe-to-close].
  *
  * New values may be added to this enum over time.
- *
- * Since: 1.0
  */
 
 typedef struct {
@@ -363,8 +356,6 @@ static void
 set_folded (AdwFlap  *self,
             gboolean  folded)
 {
-  GtkStyleContext *context;
-
   folded = !!folded;
 
   if (self->folded == folded)
@@ -386,13 +377,12 @@ set_folded (AdwFlap  *self,
   if (!self->locked)
     set_reveal_flap (self, !self->folded, 0);
 
-  context = gtk_widget_get_style_context (GTK_WIDGET (self));
   if (folded) {
-    gtk_style_context_add_class (context, "folded");
-    gtk_style_context_remove_class (context, "unfolded");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "folded");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "unfolded");
   } else {
-    gtk_style_context_remove_class (context, "folded");
-    gtk_style_context_add_class (context, "unfolded");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "folded");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "unfolded");
   }
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FOLDED]);
@@ -970,7 +960,7 @@ adw_flap_measure (GtkWidget      *widget,
     switch (self->fold_policy) {
     case ADW_FLAP_FOLD_POLICY_NEVER:
       min_progress = (1 - self->fold_progress) * self->reveal_progress;
-      nat_progress = 1;
+      nat_progress = min_progress;
       break;
 
     case ADW_FLAP_FOLD_POLICY_ALWAYS:
@@ -1268,10 +1258,7 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * The content widget.
    *
-   * It's always displayed when unfolded, and partially visible
-   * when folded.
-   *
-   * Since: 1.0
+   * It's always displayed when unfolded, and partially visible when folded.
    */
   props[PROP_CONTENT] =
     g_param_spec_object ("content", NULL, NULL,
@@ -1284,8 +1271,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * The flap widget.
    *
    * It's only visible when [property@Flap:reveal-progress] is greater than 0.
-   *
-   * Since: 1.0
    */
   props[PROP_FLAP] =
     g_param_spec_object ("flap", NULL, NULL,
@@ -1300,8 +1285,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * It's displayed between content and flap when there's no shadow to display.
    * When exactly it's visible depends on the [property@Flap:transition-type]
    * value.
-   *
-   * Since: 1.0
    */
   props[PROP_SEPARATOR] =
     g_param_spec_object ("separator", NULL, NULL,
@@ -1315,8 +1298,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * If it's set to `GTK_PACK_START`, the flap is displayed before the content,
    * if `GTK_PACK_END`, it's displayed after the content.
-   *
-   * Since: 1.0
    */
   props[PROP_FLAP_POSITION] =
     g_param_spec_enum ("flap-position", NULL, NULL,
@@ -1328,8 +1309,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * AdwFlap:reveal-flap: (attributes org.gtk.Property.get=adw_flap_get_reveal_flap org.gtk.Property.set=adw_flap_set_reveal_flap)
    *
    * Whether the flap widget is revealed.
-   *
-   * Since: 1.0
    */
   props[PROP_REVEAL_FLAP] =
     g_param_spec_boolean ("reveal-flap", NULL, NULL,
@@ -1346,8 +1325,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * ```c
    * adw_spring_params_new (1, 0.5, 500)
    * ```
-   *
-   * Since: 1.0
    */
   props[PROP_REVEAL_PARAMS] =
     g_param_spec_boxed ("reveal-params", NULL, NULL,
@@ -1362,8 +1339,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * 0 means fully hidden, 1 means fully revealed.
    *
    * See [property@Flap:reveal-flap].
-   *
-   * Since: 1.0
    */
   props[PROP_REVEAL_PROGRESS] =
     g_param_spec_double ("reveal-progress", NULL, NULL,
@@ -1374,8 +1349,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * AdwFlap:fold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_policy)
    *
    * The fold policy for the flap.
-   *
-   * Since: 1.0
    */
   props[PROP_FOLD_POLICY] =
     g_param_spec_enum ("fold-policy", NULL, NULL,
@@ -1394,8 +1367,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * This can be useful if you have a long ellipsizing label and want to let it
    * ellipsize instead of immediately folding.
-   *
-   * Since: 1.0
    */
   props[PROP_FOLD_THRESHOLD_POLICY] =
     g_param_spec_enum ("fold-threshold-policy", NULL, NULL,
@@ -1407,8 +1378,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * AdwFlap:fold-duration: (attributes org.gtk.Property.get=adw_flap_get_fold_duration org.gtk.Property.set=adw_flap_set_fold_duration)
    *
    * The fold transition animation duration, in milliseconds.
-   *
-   * Since: 1.0
    */
   props[PROP_FOLD_DURATION] =
     g_param_spec_uint ("fold-duration", NULL, NULL,
@@ -1422,8 +1391,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * Whether the flap is currently folded.
    *
    * See [property@Flap:fold-policy].
-   *
-   * Since: 1.0
    */
   props[PROP_FOLDED] =
     g_param_spec_boolean ("folded", NULL, NULL,
@@ -1438,8 +1405,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * If `FALSE`, folding when the flap is revealed automatically closes it, and
    * unfolding it when the flap is not revealed opens it. If `TRUE`,
    * [property@Flap:reveal-flap] value never changes on its own.
-   *
-   * Since: 1.0
    */
   props[PROP_LOCKED] =
     g_param_spec_boolean ("locked", NULL, NULL,
@@ -1455,8 +1420,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * will be seen through it with `ADW_FLAP_TRANSITION_TYPE_OVER` transitions;
    * add the [`.background`](style-classes.html#background) style class to it if
    * this is unwanted.
-   *
-   * Since: 1.0
    */
   props[PROP_TRANSITION_TYPE] =
     g_param_spec_enum ("transition-type", NULL, NULL,
@@ -1472,8 +1435,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    * If `TRUE`, clicking the content widget while flap is revealed, as well as
    * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks
    * are passed through to the content widget.
-   *
-   * Since: 1.0
    */
   props[PROP_MODAL] =
     g_param_spec_boolean ("modal", NULL, NULL,
@@ -1487,8 +1448,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * The area that can be swiped depends on the [property@Flap:transition-type]
    * value.
-   *
-   * Since: 1.0
    */
   props[PROP_SWIPE_TO_OPEN] =
     g_param_spec_boolean ("swipe-to-open", NULL, NULL,
@@ -1502,8 +1461,6 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * The area that can be swiped depends on the [property@Flap:transition-type]
    * value.
-   *
-   * Since: 1.0
    */
   props[PROP_SWIPE_TO_CLOSE] =
     g_param_spec_boolean ("swipe-to-close", NULL, NULL,
@@ -1533,7 +1490,6 @@ flap_close_cb (AdwFlap *self)
 static void
 adw_flap_init (AdwFlap *self)
 {
-  GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (self));
   GtkEventController *gesture;
   GtkShortcut *shortcut;
   AdwAnimationTarget *target;
@@ -1583,7 +1539,7 @@ adw_flap_init (AdwFlap *self)
 
   gtk_widget_set_overflow (GTK_WIDGET (self), GTK_OVERFLOW_HIDDEN);
 
-  gtk_style_context_add_class (context, "unfolded");
+  gtk_widget_add_css_class (GTK_WIDGET (self), "unfolded");
 
   target = adw_callback_animation_target_new ((AdwAnimationTargetFunc)
                                               fold_animation_value_cb,
@@ -1794,8 +1750,6 @@ adw_flap_swipeable_init (AdwSwipeableInterface *iface)
  * Creates a new `AdwFlap`.
  *
  * Returns: the newly created `AdwFlap`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_flap_new (void)
@@ -1810,8 +1764,6 @@ adw_flap_new (void)
  * Gets the content widget for @self.
  *
  * Returns: (transfer none) (nullable): the content widget for @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_flap_get_content (AdwFlap *self)
@@ -1828,7 +1780,7 @@ adw_flap_get_content (AdwFlap *self)
  *
  * Sets the content widget for @self.
  *
- * Since: 1.0
+ * It's always displayed when unfolded, and partially visible when folded.
  */
 void
 adw_flap_set_content (AdwFlap   *self,
@@ -1860,8 +1812,6 @@ adw_flap_set_content (AdwFlap   *self,
  * Gets the flap widget for @self.
  *
  * Returns: (transfer none) (nullable): the flap widget for @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_flap_get_flap (AdwFlap *self)
@@ -1878,7 +1828,7 @@ adw_flap_get_flap (AdwFlap *self)
  *
  * Sets the flap widget for @self.
  *
- * Since: 1.0
+ * It's only visible when [property@Flap:reveal-progress] is greater than 0.
  */
 void
 adw_flap_set_flap (AdwFlap   *self,
@@ -1911,8 +1861,6 @@ adw_flap_set_flap (AdwFlap   *self,
  * Gets the separator widget for @self.
  *
  * Returns: (transfer none) (nullable): the separator widget for @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_flap_get_separator (AdwFlap *self)
@@ -1929,7 +1877,9 @@ adw_flap_get_separator (AdwFlap *self)
  *
  * Sets the separator widget for @self.
  *
- * Since: 1.0
+ * It's displayed between content and flap when there's no shadow to display.
+ * When exactly it's visible depends on the [property@Flap:transition-type]
+ * value.
  */
 void
 adw_flap_set_separator (AdwFlap   *self,
@@ -1961,8 +1911,6 @@ adw_flap_set_separator (AdwFlap   *self,
  * Gets the flap position for @self.
  *
  * Returns: the flap position for @self
- *
- * Since: 1.0
  */
 GtkPackType
 adw_flap_get_flap_position (AdwFlap *self)
@@ -1979,7 +1927,8 @@ adw_flap_get_flap_position (AdwFlap *self)
  *
  * Sets the flap position for @self.
  *
- * Since: 1.0
+ * If it's set to `GTK_PACK_START`, the flap is displayed before the content,
+ * if `GTK_PACK_END`, it's displayed after the content.
  */
 void
 adw_flap_set_flap_position (AdwFlap     *self,
@@ -2006,8 +1955,6 @@ adw_flap_set_flap_position (AdwFlap     *self,
  * Gets whether the flap widget is revealed for @self.
  *
  * Returns: `TRUE` if the flap widget is revealed
- *
- * Since: 1.0
  */
 gboolean
 adw_flap_get_reveal_flap (AdwFlap *self)
@@ -2023,8 +1970,6 @@ adw_flap_get_reveal_flap (AdwFlap *self)
  * @reveal_flap: whether to reveal the flap widget
  *
  * Sets whether the flap widget is revealed for @self.
- *
- * Since: 1.0
  */
 void
 adw_flap_set_reveal_flap (AdwFlap  *self,
@@ -2042,8 +1987,6 @@ adw_flap_set_reveal_flap (AdwFlap  *self,
  * Gets the reveal animation spring parameters for @self.
  *
  * Returns: the reveal animation parameters
- *
- * Since: 1.0
  */
 AdwSpringParams *
 adw_flap_get_reveal_params (AdwFlap *self)
@@ -2060,7 +2003,11 @@ adw_flap_get_reveal_params (AdwFlap *self)
  *
  * Sets the reveal animation spring parameters for @self.
  *
- * Since: 1.0
+ * The default value is equivalent to:
+ *
+ * ```c
+ * adw_spring_params_new (1, 0.5, 500)
+ * ```
  */
 void
 adw_flap_set_reveal_params (AdwFlap         *self,
@@ -2084,9 +2031,11 @@ adw_flap_set_reveal_params (AdwFlap         *self,
  *
  * Gets the current reveal progress for @self.
  *
- * Returns: the current reveal progress for @self
+ * 0 means fully hidden, 1 means fully revealed.
  *
- * Since: 1.0
+ * See [property@Flap:reveal-flap].
+ *
+ * Returns: the current reveal progress for @self
  */
 double
 adw_flap_get_reveal_progress (AdwFlap *self)
@@ -2103,8 +2052,6 @@ adw_flap_get_reveal_progress (AdwFlap *self)
  * Gets the fold policy for @self.
  *
  * Returns: the fold policy for @self
- *
- * Since: 1.0
  */
 AdwFlapFoldPolicy
 adw_flap_get_fold_policy (AdwFlap *self)
@@ -2120,8 +2067,6 @@ adw_flap_get_fold_policy (AdwFlap *self)
  * @policy: the fold policy
  *
  * Sets the fold policy for @self.
- *
- * Since: 1.0
  */
 void
 adw_flap_set_fold_policy (AdwFlap           *self,
@@ -2155,15 +2100,58 @@ adw_flap_set_fold_policy (AdwFlap           *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FOLD_POLICY]);
 }
 
+/**
+ * adw_flap_get_fold_threshold_policy: (attributes org.gtk.Method.get_property=fold-threshold-policy)
+ * @self: a flap
+ *
+ * Gets the fold threshold policy for @self.
+ */
+AdwFoldThresholdPolicy
+adw_flap_get_fold_threshold_policy (AdwFlap *self)
+{
+  g_return_val_if_fail (ADW_IS_FLAP (self), ADW_FOLD_THRESHOLD_POLICY_MINIMUM);
+
+  return self->fold_threshold_policy;
+}
+
+/**
+ * adw_flap_set_fold_threshold_policy: (attributes org.gtk.Method.set_property=fold-threshold-policy)
+ * @self: a flap
+ * @policy: the policy to use
+ *
+ * Sets the fold threshold policy for @self.
+ *
+ * If set to `ADW_FOLD_THRESHOLD_POLICY_MINIMUM`, flap will only fold when the
+ * children cannot fit anymore. With `ADW_FOLD_THRESHOLD_POLICY_NATURAL`, it
+ * will fold as soon as children don't get their natural size.
+ *
+ * This can be useful if you have a long ellipsizing label and want to let it
+ * ellipsize instead of immediately folding.
+ */
+void
+adw_flap_set_fold_threshold_policy (AdwFlap                *self,
+                                    AdwFoldThresholdPolicy  policy)
+{
+  g_return_if_fail (ADW_IS_FLAP (self));
+  g_return_if_fail (policy <= ADW_FOLD_THRESHOLD_POLICY_NATURAL);
+
+  if (self->fold_threshold_policy == policy)
+    return;
+
+  self->fold_threshold_policy = policy;
+
+  gtk_widget_queue_allocate (GTK_WIDGET (self));
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FOLD_THRESHOLD_POLICY]);
+}
+
 /**
  * adw_flap_get_fold_duration: (attributes org.gtk.Method.get_property=fold-duration)
  * @self: a flap
  *
- * Gets the duration that fold transitions in @self will take.
+ * Gets the fold transition animation duration for @self, in milliseconds.
  *
  * Returns: the fold transition duration
- *
- * Since: 1.0
  */
 guint
 adw_flap_get_fold_duration (AdwFlap *self)
@@ -2178,9 +2166,7 @@ adw_flap_get_fold_duration (AdwFlap *self)
  * @self: a flap
  * @duration: the new duration, in milliseconds
  *
- * Sets the duration that fold transitions in @self will take.
- *
- * Since: 1.0
+ * Sets the fold transition animation duration for @self, in milliseconds.
  */
 void
 adw_flap_set_fold_duration (AdwFlap *self,
@@ -2202,9 +2188,9 @@ adw_flap_set_fold_duration (AdwFlap *self,
  *
  * Gets whether @self is currently folded.
  *
- * Returns: `TRUE` if @self is currently folded
+ * See [property@Flap:fold-policy].
  *
- * Since: 1.0
+ * Returns: `TRUE` if @self is currently folded
  */
 gboolean
 adw_flap_get_folded (AdwFlap *self)
@@ -2221,8 +2207,6 @@ adw_flap_get_folded (AdwFlap *self)
  * Gets whether @self is locked.
  *
  * Returns: `TRUE` if @self is locked
- *
- * Since: 1.0
  */
 gboolean
 adw_flap_get_locked (AdwFlap *self)
@@ -2239,7 +2223,9 @@ adw_flap_get_locked (AdwFlap *self)
  *
  * Sets whether @self is locked.
  *
- * Since: 1.0
+ * If `FALSE`, folding when the flap is revealed automatically closes it, and
+ * unfolding it when the flap is not revealed opens it. If `TRUE`,
+ * [property@Flap:reveal-flap] value never changes on its own.
  */
 void
 adw_flap_set_locked (AdwFlap  *self,
@@ -2264,8 +2250,6 @@ adw_flap_set_locked (AdwFlap  *self,
  * Gets the type of animation used for reveal and fold transitions in @self.
  *
  * Returns: the current transition type of @self
- *
- * Since: 1.0
  */
 AdwFlapTransitionType
 adw_flap_get_transition_type (AdwFlap *self)
@@ -2282,7 +2266,10 @@ adw_flap_get_transition_type (AdwFlap *self)
  *
  * Sets the type of animation used for reveal and fold transitions in @self.
  *
- * Since: 1.0
+ * [property@Flap:flap] is transparent by default, which means the content will
+ * be seen through it with `ADW_FLAP_TRANSITION_TYPE_OVER` transitions; add the
+ * [`.background`](style-classes.html#background) style class to it if this is
+ * unwanted.
  */
 void
 adw_flap_set_transition_type (AdwFlap               *self,
@@ -2311,8 +2298,6 @@ adw_flap_set_transition_type (AdwFlap               *self,
  * Gets whether @self is modal.
  *
  * Returns: `TRUE` if @self is modal
- *
- * Since: 1.0
  */
 gboolean
 adw_flap_get_modal (AdwFlap *self)
@@ -2329,7 +2314,9 @@ adw_flap_get_modal (AdwFlap *self)
  *
  * Sets whether @self is modal.
  *
- * Since: 1.0
+ * If `TRUE`, clicking the content widget while flap is revealed, as well as
+ * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks are
+ * passed through to the content widget.
  */
 void
 adw_flap_set_modal (AdwFlap  *self,
@@ -2359,8 +2346,6 @@ adw_flap_set_modal (AdwFlap  *self,
  * Gets whether @self can be opened with a swipe gesture.
  *
  * Returns: `TRUE` if @self can be opened with a swipe gesture
- *
- * Since: 1.0
  */
 gboolean
 adw_flap_get_swipe_to_open (AdwFlap *self)
@@ -2377,7 +2362,8 @@ adw_flap_get_swipe_to_open (AdwFlap *self)
  *
  * Sets whether @self can be opened with a swipe gesture.
  *
- * Since: 1.0
+ * The area that can be swiped depends on the [property@Flap:transition-type]
+ * value.
  */
 void
 adw_flap_set_swipe_to_open (AdwFlap  *self,
@@ -2404,8 +2390,6 @@ adw_flap_set_swipe_to_open (AdwFlap  *self,
  * Gets whether @self can be closed with a swipe gesture.
  *
  * Returns: `TRUE` if @self can be closed with a swipe gesture
- *
- * Since: 1.0
  */
 gboolean
 adw_flap_get_swipe_to_close (AdwFlap *self)
@@ -2422,7 +2406,8 @@ adw_flap_get_swipe_to_close (AdwFlap *self)
  *
  * Sets whether @self can be closed with a swipe gesture.
  *
- * Since: 1.0
+ * The area that can be swiped depends on the [property@Flap:transition-type]
+ * value.
  */
 void
 adw_flap_set_swipe_to_close (AdwFlap  *self,
@@ -2441,45 +2426,3 @@ adw_flap_set_swipe_to_close (AdwFlap  *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SWIPE_TO_CLOSE]);
 }
-
-/**
- * adw_flap_get_fold_threshold_policy: (attributes org.gtk.Method.get_property=fold-threshold-policy)
- * @self: a flap
- *
- * Gets the fold threshold policy for @self.
- *
- * Since: 1.0
- */
-AdwFoldThresholdPolicy
-adw_flap_get_fold_threshold_policy (AdwFlap *self)
-{
-  g_return_val_if_fail (ADW_IS_FLAP (self), ADW_FOLD_THRESHOLD_POLICY_MINIMUM);
-
-  return self->fold_threshold_policy;
-}
-
-/**
- * adw_flap_set_fold_threshold_policy: (attributes org.gtk.Method.set_property=fold-threshold-policy)
- * @self: a flap
- * @policy: the policy to use
- *
- * Sets the fold threshold policy for @self.
- *
- * Since: 1.0
- */
-void
-adw_flap_set_fold_threshold_policy (AdwFlap                *self,
-                                    AdwFoldThresholdPolicy  policy)
-{
-  g_return_if_fail (ADW_IS_FLAP (self));
-  g_return_if_fail (policy <= ADW_FOLD_THRESHOLD_POLICY_NATURAL);
-
-  if (self->fold_threshold_policy == policy)
-    return;
-
-  self->fold_threshold_policy = policy;
-
-  gtk_widget_queue_allocate (GTK_WIDGET (self));
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FOLD_THRESHOLD_POLICY]);
-}
