@@ -9,8 +9,9 @@
 #include "config.h"
 #include "adw-message-dialog.h"
 
+#include "adw-gizmo-private.h"
 #include "adw-gtkbuilder-utils-private.h"
-#include "adw-squeezer.h"
+#include "adw-widget-utils-private.h"
 
 /**
  * AdwMessageDialog:
@@ -67,6 +68,48 @@
  * gtk_window_present (GTK_WINDOW (dialog));
  * ```
  *
+ * ## Async API
+ *
+ * `AdwMessageDialog` can also be used via the [method@MessageDialog.choose]
+ * method. This API follows the GIO async pattern, and the result can be
+ * obtained by calling [method@MessageDialog.choose_finish], for example:
+ *
+ * ```c
+ * static void
+ * dialog_cb (AdwMessageDialog *dialog,
+ *            GAsyncResult     *result,
+ *            MyWindow         *self)
+ * {
+ *   const char *response = adw_message_dialog_choose_finish (dialog, result);
+ *
+ *   ...
+ * }
+ *
+ * static void
+ * show_dialog (MyWindow *self)
+ * {
+ *   GtkWidget *dialog;
+ *
+ *   dialog = adw_message_dialog_new (GTK_WINDOW (self), _("Replace File?"), NULL);
+ *
+ *   adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog),
+ *                                   _("A file named “%s” already exists. Do you want to replace it?"),
+ *                                   filename);
+ *
+ *   adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+ *                                     "cancel",  _("_Cancel"),
+ *                                     "replace", _("_Replace"),
+ *                                     NULL);
+ *
+ *   adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dialog), "replace", ADW_RESPONSE_DESTRUCTIVE);
+ *
+ *   adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
+ *   adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
+ *
+ *   adw_message_dialog_choose (ADW_MESSAGE_DIALOG (dialog), NULL, (GAsyncReadyCallback) dialog_cb, self);
+ * }
+ * ```
+ *
  * ## AdwMessageDialog as GtkBuildable
  *
  * `AdwMessageDialog` supports adding responses in UI definitions by via the
@@ -91,7 +134,7 @@
  *   <property name="body" translatable="yes">Open documents contain unsaved changes. Changes which are not saved will be permanently lost.</property>
  *   <property name="default-response">save</property>
  *   <property name="close-response">cancel</property>
- *   <signal name="response" handler="response_id"/>
+ *   <signal name="response" handler="response_cb"/>
  *   <responses>
  *     <response id="cancel" translatable="yes">_Cancel</response>
  *     <response id="discard" translatable="yes" appearance="destructive">_Discard</response>
@@ -134,8 +177,8 @@ typedef struct {
   AdwResponseAppearance appearance;
   gboolean enabled;
 
-  GtkWidget *wide_button;
-  GtkWidget *narrow_button;
+  GtkWidget *button;
+  GtkWidget *separator;
 } ResponseInfo;
 
 typedef struct
@@ -143,11 +186,7 @@ typedef struct
   GtkWidget *heading_label;
   GtkWidget *body_label;
   GtkBox *message_area;
-  GtkBox *wide_response_box;
-  GtkBox *narrow_response_box;
-  GtkSizeGroup *wide_size_group;
-  GtkSizeGroup *narrow_size_group;
-  AdwSqueezer *squeezer;
+  GtkWidget *response_area;
 
   char *heading;
   gboolean heading_use_markup;
@@ -264,6 +303,16 @@ parent_unrealize_cb (AdwMessageDialog *self)
   priv->parent_height = -1;
 }
 
+static void
+parent_window_notify_cb (AdwMessageDialog *self)
+{
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+
+  priv->parent_window = NULL;
+  priv->parent_width = -1;
+  priv->parent_height = -1;
+}
+
 static void
 set_parent (AdwMessageDialog *self,
             GtkWindow        *parent)
@@ -283,11 +332,19 @@ set_parent (AdwMessageDialog *self,
 
     if (gtk_widget_get_realized (GTK_WIDGET (priv->parent_window)))
       parent_unrealize_cb (self);
+
+    g_object_weak_unref (G_OBJECT (priv->parent_window),
+                         (GWeakNotify) parent_window_notify_cb,
+                         self);
   }
 
   priv->parent_window = parent;
 
   if (priv->parent_window) {
+    g_object_weak_ref (G_OBJECT (priv->parent_window),
+                       (GWeakNotify) parent_window_notify_cb,
+                       self);
+
     if (gtk_widget_get_realized (GTK_WIDGET (priv->parent_window)))
       parent_realize_cb (self);
 
@@ -349,26 +406,6 @@ create_response_button (AdwMessageDialog *self,
   return button;
 }
 
-static void
-update_default_response (AdwMessageDialog *self)
-{
-  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
-  ResponseInfo *info;
-
-  if (!priv->default_response)
-    return;
-
-  info = find_response (self, g_quark_to_string (priv->default_response));
-
-  if (!info)
-    return;
-
-  if (adw_squeezer_get_visible_child (priv->squeezer) == GTK_WIDGET (priv->narrow_response_box))
-    gtk_window_set_default_widget (GTK_WINDOW (self), info->narrow_button);
-  else
-    gtk_window_set_default_widget (GTK_WINDOW (self), info->wide_button);
-}
-
 static void
 update_window_title (AdwMessageDialog *self)
 {
@@ -421,8 +458,6 @@ adw_message_dialog_map (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (adw_message_dialog_parent_class)->map (widget);
 
-  update_default_response (self);
-
   /* The rest of the function was copied from gtkdialog.c */
   focus = gtk_window_get_focus (GTK_WINDOW (self));
   if (!focus) {
@@ -450,19 +485,210 @@ adw_message_dialog_map (GtkWidget *widget)
     for (l = priv->responses; l; l = l->next) {
       ResponseInfo *response = l->data;
 
-      if ((focus == NULL || response->wide_button == focus) &&
-           response->wide_button != default_widget &&
+      if ((focus == NULL || response->button == focus) &&
+           response->button != default_widget &&
            default_widget) {
         gtk_widget_grab_focus (default_widget);
         break;
       }
+    }
+  }
+}
+
+static void
+measure_responses_do (AdwMessageDialog *self,
+                      gboolean          compact,
+                      GtkOrientation    orientation,
+                      int              *minimum,
+                      int              *natural)
+{
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  GList *l;
+  int min = 0, nat = 0;
+  int button_min = 0, button_nat = 0;
+  int n_buttons = 0;
+  gboolean horiz = (orientation == GTK_ORIENTATION_HORIZONTAL);
+
+  for (l = priv->responses; l; l = l->next) {
+    ResponseInfo *response = l->data;
+    int child_min, child_nat;
+
+    gtk_widget_measure (response->button, orientation, -1,
+                        &child_min, &child_nat, NULL, NULL);
+
+    if (horiz == compact) {
+      min = MAX (min, child_min);
+      nat = MAX (nat, child_min);
+    } else if (horiz) {
+      button_min = MAX (button_min, child_min);
+      button_nat = MAX (button_nat, child_min);
+      n_buttons++;
+    } else {
+      min += child_min;
+      nat += child_nat;
+    }
+
+    if (response->separator) {
+      gtk_widget_measure (response->separator, orientation, -1,
+                          &child_min, &child_nat, NULL, NULL);
+
+    if (horiz == compact) {
+        min = MAX (min, child_min);
+        nat = MAX (nat, child_min);
+      } else {
+        min += child_min;
+        nat += child_nat;
+      }
+    }
+  }
+
+  if (horiz && !compact) {
+    min += button_min * n_buttons;
+    nat += button_nat * n_buttons;
+  }
+
+  if (minimum)
+    *minimum = min;
+  if (natural)
+    *natural = nat;
+}
+
+static GtkSizeRequestMode
+get_responses_request_mode (GtkWidget *widget)
+{
+  return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+}
+
+static void
+measure_responses (GtkWidget      *widget,
+                   GtkOrientation  orientation,
+                   int             for_size,
+                   int            *minimum,
+                   int            *natural,
+                   int            *minimum_baseline,
+                   int            *natural_baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+    measure_responses_do (self, TRUE, orientation, minimum, NULL);
+    measure_responses_do (self, FALSE, orientation, NULL, natural);
+  } else {
+    int wide_min = 0;
+
+    if (for_size >= 0)
+      measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+
+    measure_responses_do (self, for_size >= 0 && for_size < wide_min,
+                          orientation, minimum, natural);
+  }
+
+  if (minimum_baseline)
+    *minimum_baseline = -1;
+  if (natural_baseline)
+    *natural_baseline = -1;
+}
+
+static void
+allocate_responses (GtkWidget *widget,
+                    int        width,
+                    int        height,
+                    int        baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  gboolean compact;
+  int wide_min;
+
+  measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+
+  compact = wide_min > width;
+
+  if (compact)
+    gtk_widget_add_css_class (widget, "compact");
+  else
+    gtk_widget_remove_css_class (widget, "compact");
+
+  if (compact) {
+    int pos = height;
+    GList *l;
+
+    for (l = priv->responses; l; l = l->next) {
+      ResponseInfo *response = l->data;
+      int child_height;
+
+      if (response->separator) {
+        gtk_widget_measure (response->separator, GTK_ORIENTATION_VERTICAL, -1,
+                            &child_height, NULL, NULL, NULL);
+
+        pos -= child_height;
+
+        gtk_widget_allocate (response->separator, width, child_height, -1,
+                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
+      }
 
-      if ((focus == NULL || response->narrow_button == focus) &&
-           response->narrow_button != default_widget &&
-           default_widget) {
-        gtk_widget_grab_focus (default_widget);
-        break;
+      gtk_widget_measure (response->button, GTK_ORIENTATION_VERTICAL, -1,
+                          &child_height, NULL, NULL, NULL);
+
+      pos -= child_height;
+
+      gtk_widget_allocate (response->button, width, child_height, -1,
+                           gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
+    }
+  } else {
+    gboolean is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
+    int pos = is_rtl ? width : 0;
+    int n_buttons = g_list_length (priv->responses);
+    int total_width = width;
+    int button_width;
+    GList *l;
+
+    for (l = priv->responses; l; l = l->next) {
+      ResponseInfo *response = l->data;
+      int separator_width;
+
+      if (!response->separator)
+        continue;
+
+      gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
+                          &separator_width, NULL, NULL, NULL);
+
+      total_width -= separator_width;
+    }
+
+    button_width = (int) ceil ((double) total_width / n_buttons);
+
+    for (l = priv->responses; l; l = l->next) {
+      ResponseInfo *response = l->data;
+
+      if (response->separator) {
+        int separator_width;
+
+        gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
+                            &separator_width, NULL, NULL, NULL);
+
+        if (is_rtl)
+          pos -= separator_width;
+
+        gtk_widget_allocate (response->separator, separator_width, height, -1,
+                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
+
+        if (!is_rtl)
+          pos += separator_width;
       }
+
+      button_width = MIN (button_width, total_width);
+
+      total_width -= button_width;
+
+      if (is_rtl)
+        pos -= button_width;
+
+      gtk_widget_allocate (response->button, button_width, height, -1,
+                           gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
+
+      if (!is_rtl)
+        pos += button_width;
     }
   }
 }
@@ -492,15 +718,15 @@ adw_message_dialog_measure (GtkWidget      *widget,
 
     min_size = MAX (min_size, DIALOG_MIN_WIDTH);
 
-    max_size = priv->parent_width - DIALOG_MARGIN * 2;
-    max_size = MIN (max_size, DIALOG_MAX_WIDTH);
+    if (priv->parent_window) {
+      max_size = priv->parent_width - DIALOG_MARGIN * 2;
+      max_size = MIN (max_size, DIALOG_MAX_WIDTH);
+    } else {
+      max_size = DIALOG_MAX_WIDTH;
+    }
 
-    gtk_widget_measure (GTK_WIDGET (priv->wide_response_box),
-                        GTK_ORIENTATION_HORIZONTAL, -1,
-                        &wide_min, NULL, NULL, NULL);
-    gtk_widget_measure (GTK_WIDGET (priv->narrow_response_box),
-                        GTK_ORIENTATION_HORIZONTAL, -1,
-                        NULL, &narrow_nat, NULL, NULL);
+    measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+    measure_responses_do (self, TRUE, GTK_ORIENTATION_HORIZONTAL, NULL, &narrow_nat);
 
     narrow_nat = MAX (narrow_nat, DIALOG_MIN_WIDTH);
 
@@ -780,17 +1006,13 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_label);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, body_label);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, message_area);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, squeezer);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, wide_response_box);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, narrow_response_box);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, wide_size_group);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, narrow_size_group);
-
-  gtk_widget_class_bind_template_callback (widget_class, update_default_response);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, response_area);
 
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_DIALOG);
+
+  g_type_ensure (ADW_TYPE_GIZMO);
 }
 
 static void
@@ -812,6 +1034,16 @@ adw_message_dialog_init (AdwMessageDialog *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  gtk_widget_set_layout_manager (priv->response_area,
+                                 gtk_custom_layout_new (get_responses_request_mode,
+                                                        measure_responses,
+                                                        allocate_responses));
+
+  adw_gizmo_set_focus_func (ADW_GIZMO (priv->response_area),
+                            (AdwGizmoFocusFunc) adw_widget_focus_child);
+  adw_gizmo_set_grab_focus_func (ADW_GIZMO (priv->response_area),
+                            (AdwGizmoGrabFocusFunc) adw_widget_grab_focus_child);
+
   parent_changed_cb (self);
   g_signal_connect (self, "notify::transient-for",
                     G_CALLBACK (parent_changed_cb), self);
@@ -1611,30 +1843,18 @@ adw_message_dialog_add_response (AdwMessageDialog *self,
   info->enabled = TRUE;
 
   if (priv->responses) {
-    GtkWidget *separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-    gtk_box_append (priv->wide_response_box, separator);
+    info->separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
+    gtk_widget_set_parent (info->separator, priv->response_area);
   }
 
-  info->wide_button = create_response_button (self, info);
-  gtk_widget_set_hexpand (info->wide_button, TRUE);
-  gtk_box_append (priv->wide_response_box, info->wide_button);
-  gtk_size_group_add_widget (priv->wide_size_group, info->wide_button);
-
-  if (priv->responses) {
-    GtkWidget *separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-    gtk_box_prepend (priv->narrow_response_box, separator);
-  }
-
-  info->narrow_button = create_response_button (self, info);
-  gtk_box_prepend (priv->narrow_response_box, info->narrow_button);
-  gtk_size_group_add_widget (priv->narrow_size_group, info->narrow_button);
+  info->button = create_response_button (self, info);
+  gtk_widget_set_parent (info->button, priv->response_area);
 
   priv->responses = g_list_append (priv->responses, info);
   g_hash_table_insert (priv->id_to_response, g_strdup (id), info);
 
-  if (priv->default_response == info->id &&
-      gtk_widget_get_mapped (GTK_WIDGET (self)))
-    update_default_response (self);
+  if (priv->default_response == info->id)
+    gtk_window_set_default_widget (GTK_WINDOW (self), info->button);
 }
 
 /**
@@ -1750,8 +1970,7 @@ adw_message_dialog_set_response_label (AdwMessageDialog *self,
   g_free (info->label);
   info->label = g_strdup (label);
 
-  gtk_button_set_label (GTK_BUTTON (info->wide_button), label);
-  gtk_button_set_label (GTK_BUTTON (info->narrow_button), label);
+  gtk_button_set_label (GTK_BUTTON (info->button), label);
 }
 
 /**
@@ -1828,21 +2047,15 @@ adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
 
   info->appearance = appearance;
 
-  if (info->appearance == ADW_RESPONSE_SUGGESTED) {
-    gtk_widget_add_css_class (info->wide_button, "suggested");
-    gtk_widget_add_css_class (info->narrow_button, "suggested");
-  } else {
-    gtk_widget_remove_css_class (info->wide_button, "suggested");
-    gtk_widget_remove_css_class (info->narrow_button, "suggested");
-  }
+  if (info->appearance == ADW_RESPONSE_SUGGESTED)
+    gtk_widget_add_css_class (info->button, "suggested");
+  else
+    gtk_widget_remove_css_class (info->button, "suggested");
 
-  if (info->appearance == ADW_RESPONSE_DESTRUCTIVE) {
-    gtk_widget_add_css_class (info->wide_button, "destructive");
-    gtk_widget_add_css_class (info->narrow_button, "destructive");
-  } else {
-    gtk_widget_remove_css_class (info->wide_button, "destructive");
-    gtk_widget_remove_css_class (info->narrow_button, "destructive");
-  }
+  if (info->appearance == ADW_RESPONSE_DESTRUCTIVE)
+    gtk_widget_add_css_class (info->button, "destructive");
+  else
+    gtk_widget_remove_css_class (info->button, "destructive");
 }
 
 /**
@@ -1912,8 +2125,7 @@ adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
 
   info->enabled = enabled;
 
-  gtk_widget_set_sensitive (info->wide_button, info->enabled);
-  gtk_widget_set_sensitive (info->narrow_button, info->enabled);
+  gtk_widget_set_sensitive (info->button, info->enabled);
 }
 
 /**
@@ -1961,6 +2173,7 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
 {
   AdwMessageDialogPrivate *priv;
   GQuark quark;
+  ResponseInfo *info;
 
   g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
 
@@ -1972,8 +2185,10 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
 
   priv->default_response = quark;
 
-  if (gtk_widget_get_mapped (GTK_WIDGET (self)))
-    update_default_response (self);
+  info = find_response (self, response);
+
+  if (info)
+    gtk_window_set_default_widget (GTK_WINDOW (self), info->button);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DEFAULT_RESPONSE]);
 }
@@ -2079,3 +2294,91 @@ adw_message_dialog_has_response (AdwMessageDialog *self,
 
   return find_response (self, response) != NULL;
 }
+
+static void choose_cancelled_cb (GCancellable *cancellable,
+                                 GTask        *task);
+
+static void
+choose_response_cb (AdwMessageDialog *dialog,
+                    const char       *response,
+                    GTask            *task)
+{
+  GCancellable *cancellable = g_task_get_cancellable (task);
+
+  if (cancellable)
+    g_signal_handlers_disconnect_by_func (cancellable, choose_cancelled_cb, task);
+
+  g_task_return_int (task, g_quark_from_string (response));
+
+  g_object_unref (task);
+}
+
+static void
+choose_cancelled_cb (GCancellable *cancellable,
+                     GTask        *task)
+{
+  AdwMessageDialog *self = g_task_get_source_object (task);
+
+  choose_response_cb (self, adw_message_dialog_get_close_response (self), task);
+}
+
+/**
+ * adw_message_dialog_choose:
+ * @self: (transfer full): a message dialog
+ * @cancellable: (nullable): a `GCancellable` to cancel the operation
+ * @callback: (scope async): a callback to call when the operation is complete
+ * @user_data: (closure callback): data to pass to @callback
+ *
+ * This function shows @self to the user.
+ *
+ * The @callback will be called when the alert is dismissed. It should call
+ * [method@MessageDialog.choose_finish] to obtain the result.
+ *
+ * Since: 1.3
+ */
+void
+adw_message_dialog_choose (AdwMessageDialog    *self,
+                           GCancellable        *cancellable,
+                           GAsyncReadyCallback  callback,
+                           gpointer             user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, adw_message_dialog_choose);
+
+  if (cancellable)
+    g_signal_connect (cancellable, "cancelled", G_CALLBACK (choose_cancelled_cb), task);
+
+  g_signal_connect (self, "response", G_CALLBACK (choose_response_cb), task);
+
+  gtk_window_present (GTK_WINDOW (self));
+}
+
+/**
+ * adw_message_dialog_choose_finish:
+ * @self: a message dialog
+ * @result: a `GAsyncResult`
+ *
+ * Finishes the [method@MessageDialog.choose] call and returns the response ID.
+ *
+ * Returns: the ID of the response that was selected, or
+ *   [property@MessageDialog:close-response] if the call was cancelled.
+ *
+ * Since: 1.3
+ */
+const char *
+adw_message_dialog_choose_finish (AdwMessageDialog *self,
+                                  GAsyncResult     *result)
+{
+  GQuark id;
+  g_return_val_if_fail (ADW_IS_MESSAGE_DIALOG (self), NULL);
+  g_return_val_if_fail (g_task_is_valid (result, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == adw_message_dialog_choose, NULL);
+
+  id = g_task_propagate_int (G_TASK (result), NULL);
+
+  return g_quark_to_string (id);
+}
