@@ -8,7 +8,6 @@
 
 #include "adw-split-button.h"
 
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -50,8 +49,6 @@
  * ## Accessibility
  *
  * `AdwSplitButton` uses the `GTK_ACCESSIBLE_ROLE_GROUP` role.
- *
- * Since: 1.0
  */
 
 enum {
@@ -63,6 +60,7 @@ enum {
   PROP_MENU_MODEL,
   PROP_POPOVER,
   PROP_DIRECTION,
+  PROP_DROPDOWN_TOOLTIP,
 
   /* actionable properties */
   PROP_ACTION_NAME,
@@ -192,6 +190,9 @@ adw_split_button_get_property (GObject    *object,
   case PROP_DIRECTION:
     g_value_set_enum (value, adw_split_button_get_direction (self));
     break;
+  case PROP_DROPDOWN_TOOLTIP:
+    g_value_set_string (value, adw_split_button_get_dropdown_tooltip (self));
+    break;
   case PROP_ACTION_NAME:
     g_value_set_string (value, gtk_actionable_get_action_name (GTK_ACTIONABLE (self)));
     break;
@@ -234,6 +235,9 @@ adw_split_button_set_property (GObject      *object,
   case PROP_DIRECTION:
     adw_split_button_set_direction (self, g_value_get_enum (value));
     break;
+  case PROP_DROPDOWN_TOOLTIP:
+    adw_split_button_set_dropdown_tooltip (self, g_value_get_string (value));
+    break;
   case PROP_ACTION_NAME:
     gtk_actionable_set_action_name (GTK_ACTIONABLE (self), g_value_get_string (value));
     break;
@@ -281,8 +285,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * Setting the label will set [property@SplitButton:icon-name] and
    * [property@SplitButton:child] to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_LABEL] =
     g_param_spec_string ("label", NULL, NULL,
@@ -295,8 +297,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    * Whether an underline in the text indicates a mnemonic.
    *
    * See [property@SplitButton:label].
-   *
-   * Since: 1.0
    */
   props[PROP_USE_UNDERLINE] =
     g_param_spec_boolean ("use-underline", NULL, NULL,
@@ -310,8 +310,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * Setting the icon name will set [property@SplitButton:label] and
    * [property@SplitButton:child] to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -325,8 +323,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * Setting the child widget will set [property@SplitButton:label] and
    * [property@SplitButton:icon-name] to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -341,13 +337,11 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    * If the menu model is `NULL`, the dropdown is disabled.
    *
    * A [class@Gtk.Popover] will be created from the menu model with
-   * [ctor@Gtk.PopoverMenu.new_from_model]. Actions will be connected
-   * as documented for this function.
+   * [ctor@Gtk.PopoverMenu.new_from_model]. Actions will be connected as
+   * documented for this function.
    *
    * If [property@SplitButton:popover] is already set, it will be dissociated
    * from the button, and the property is set to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_MENU_MODEL] =
     g_param_spec_object ("menu-model", NULL, NULL,
@@ -363,8 +357,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * If [property@SplitButton:menu-model] is set, the menu model is dissociated
    * from the button, and the property is set to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_POPOVER] =
     g_param_spec_object ("popover", NULL, NULL,
@@ -378,12 +370,10 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * The dropdown arrow icon will point at the same direction.
    *
-   * If the does not fit in the available space in the given direction,
-   * GTK will its best to keep it inside the screen and fully visible.
+   * If the does not fit in the available space in the given direction, GTK will
+   * try its best to keep it inside the screen and fully visible.
    *
    * If you pass `GTK_ARROW_NONE`, it's equivalent to `GTK_ARROW_DOWN`.
-   *
-   * Since: 1.0
    */
   props[PROP_DIRECTION] =
     g_param_spec_enum ("direction", NULL, NULL,
@@ -391,6 +381,20 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
                        GTK_ARROW_DOWN,
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwSplitButton:dropdown-tooltip: (attributes org.gtk.Property.get=adw_split_button_get_dropdown_tooltip org.gtk.Property.set=adw_split_button_set_dropdown_tooltip)
+   *
+   * The tooltip of the dropdown button.
+   *
+   * The tooltip can be marked up with the Pango text markup language.
+   *
+   * Since: 1.2
+   */
+  props[PROP_DROPDOWN_TOOLTIP] =
+    g_param_spec_string ("dropdown-tooltip", NULL, NULL,
+                         "",
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   g_object_class_override_property (object_class, PROP_ACTION_NAME, "action-name");
@@ -400,8 +404,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    * AdwSplitButton::clicked:
    *
    * Emitted when the button has been activated (pressed and released).
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_CLICKED] =
     g_signal_new ("clicked",
@@ -418,8 +420,6 @@ adw_split_button_class_init (AdwSplitButtonClass *klass)
    *
    * This is an action signal. Applications should never connect to this signal,
    * but use the [signal@SplitButton::clicked] signal.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_ACTIVATE] =
     g_signal_new ("activate",
@@ -495,10 +495,6 @@ adw_split_button_init (AdwSplitButton *self)
   g_signal_connect_swapped (self->menu_button, "notify::popover", G_CALLBACK (notify_popover_cb), self);
   g_signal_connect_swapped (self->menu_button, "notify::direction", G_CALLBACK (notify_direction_cb), self);
 
-  g_object_bind_property (self->button, "sensitive",
-                          self, "sensitive",
-                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-
   update_style_classes (self);
 }
 
@@ -573,8 +569,6 @@ adw_split_button_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwSplitButton`.
  *
  * Returns: the newly created `AdwSplitButton`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_split_button_new (void)
@@ -589,8 +583,6 @@ adw_split_button_new (void)
  * Gets the label for @self.
  *
  * Returns: (nullable): the label for @self
- *
- * Since: 1.0
  */
 const char *
 adw_split_button_get_label (AdwSplitButton *self)
@@ -607,7 +599,8 @@ adw_split_button_get_label (AdwSplitButton *self)
  *
  * Sets the label for @self.
  *
- * Since: 1.0
+ * Setting the label will set [property@SplitButton:icon-name] and
+ * [property@SplitButton:child] to `NULL`.
  */
 void
 adw_split_button_set_label (AdwSplitButton *self,
@@ -639,8 +632,6 @@ adw_split_button_set_label (AdwSplitButton *self,
  * Gets whether an underline in the text indicates a mnemonic.
  *
  * Returns: whether an underline in the text indicates a mnemonic
- *
- * Since: 1.0
  */
 gboolean
 adw_split_button_get_use_underline (AdwSplitButton *self)
@@ -657,7 +648,7 @@ adw_split_button_get_use_underline (AdwSplitButton *self)
  *
  * Sets whether an underline in the text indicates a mnemonic.
  *
- * Since: 1.0
+ * See [property@SplitButton:label].
  */
 void
 adw_split_button_set_use_underline (AdwSplitButton *self,
@@ -679,12 +670,7 @@ adw_split_button_set_use_underline (AdwSplitButton *self,
  *
  * Gets the name of the icon used to automatically populate the button.
  *
- * If the icon name has not been set with [method@SplitButton.set_icon_name]
- * the return value will be `NULL`.
- *
  * Returns: (nullable): the icon name
- *
- * Since: 1.0
  */
 const char *
 adw_split_button_get_icon_name (AdwSplitButton *self)
@@ -701,7 +687,8 @@ adw_split_button_get_icon_name (AdwSplitButton *self)
  *
  * Sets the name of the icon used to automatically populate the button.
  *
- * Since: 1.0
+ * Setting the icon name will set [property@SplitButton:label] and
+ * [property@SplitButton:child] to `NULL`.
  */
 void
 adw_split_button_set_icon_name (AdwSplitButton *self,
@@ -734,8 +721,6 @@ adw_split_button_set_icon_name (AdwSplitButton *self,
  * Gets the child widget.
  *
  * Returns: (transfer none) (nullable): the child widget
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_split_button_get_child (AdwSplitButton *self)
@@ -752,7 +737,8 @@ adw_split_button_get_child (AdwSplitButton *self)
  *
  * Sets the child widget.
  *
- * Since: 1.0
+ * Setting the child widget will set [property@SplitButton:label] and
+ * [property@SplitButton:icon-name] to `NULL`.
  */
 void
 adw_split_button_set_child (AdwSplitButton *self,
@@ -784,8 +770,6 @@ adw_split_button_set_child (AdwSplitButton *self,
  * Gets the menu model from which the popup will be created.
  *
  * Returns: (transfer none) (nullable): the menu model
- *
- * Since: 1.0
  */
 GMenuModel *
 adw_split_button_get_menu_model (AdwSplitButton *self)
@@ -802,7 +786,14 @@ adw_split_button_get_menu_model (AdwSplitButton *self)
  *
  * Sets the menu model from which the popup will be created.
  *
- * Since: 1.0
+ * If the menu model is `NULL`, the dropdown is disabled.
+ *
+ * A [class@Gtk.Popover] will be created from the menu model with
+ * [ctor@Gtk.PopoverMenu.new_from_model]. Actions will be connected as
+ * documented for this function.
+ *
+ * If [property@SplitButton:popover] is already set, it will be dissociated from
+ * the button, and the property is set to `NULL`.
  */
 void
 adw_split_button_set_menu_model (AdwSplitButton *self,
@@ -823,8 +814,6 @@ adw_split_button_set_menu_model (AdwSplitButton *self,
  * Gets the popover that will be popped up when the dropdown is clicked.
  *
  * Returns: (transfer none) (nullable): the popover
- *
- * Since: 1.0
  */
 GtkPopover *
 adw_split_button_get_popover (AdwSplitButton *self)
@@ -841,7 +830,10 @@ adw_split_button_get_popover (AdwSplitButton *self)
  *
  * Sets the popover that will be popped up when the dropdown is clicked.
  *
- * Since: 1.0
+ * If the popover is `NULL`, the dropdown is disabled.
+ *
+ * If [property@SplitButton:menu-model] is set, the menu model is dissociated
+ * from the button, and the property is set to `NULL`.
  */
 void
 adw_split_button_set_popover (AdwSplitButton *self,
@@ -862,8 +854,6 @@ adw_split_button_set_popover (AdwSplitButton *self,
  * Gets the direction in which the popup will be popped up.
  *
  * Returns: the direction
- *
- * Since: 1.0
  */
 GtkArrowType
 adw_split_button_get_direction (AdwSplitButton *self)
@@ -880,7 +870,12 @@ adw_split_button_get_direction (AdwSplitButton *self)
  *
  * Sets the direction in which the popup will be popped up.
  *
- * Since: 1.0
+ * The dropdown arrow icon will point at the same direction.
+ *
+ * If the does not fit in the available space in the given direction, GTK will
+ * try its best to keep it inside the screen and fully visible.
+ *
+ * If you pass `GTK_ARROW_NONE`, it's equivalent to `GTK_ARROW_DOWN`.
  */
 void
 adw_split_button_set_direction (AdwSplitButton *self,
@@ -896,13 +891,59 @@ adw_split_button_set_direction (AdwSplitButton *self,
   update_style_classes (self);
 }
 
+/**
+ * adw_split_button_get_dropdown_tooltip: (attributes org.gtk.Method.get_property=dropdown-tooltip)
+ * @self: a split button
+ *
+ * Gets the tooltip of the dropdown button of @self.
+ *
+ * Returns: (transfer none): the dropdown tooltip of @self
+ *
+ * Since: 1.2
+ */
+const char *
+adw_split_button_get_dropdown_tooltip (AdwSplitButton *self)
+{
+  const char *tooltip;
+
+  g_return_val_if_fail (ADW_IS_SPLIT_BUTTON (self), NULL);
+
+  tooltip = gtk_widget_get_tooltip_markup (self->menu_button);
+
+  return tooltip ? tooltip : "";
+}
+
+/**
+ * adw_split_button_set_dropdown_tooltip: (attributes org.gtk.Method.set_property=dropdown-tooltip)
+ * @self: a split button
+ * @tooltip: the dropdown tooltip of @self
+ *
+ * Sets the tooltip of the dropdown button of @self.
+ *
+ * The tooltip can be marked up with the Pango text markup language.
+ *
+ * Since: 1.2
+ */
+void
+adw_split_button_set_dropdown_tooltip (AdwSplitButton *self,
+                                       const char     *tooltip)
+{
+  g_return_if_fail (ADW_IS_SPLIT_BUTTON (self));
+  g_return_if_fail (tooltip != NULL);
+
+  if (!g_strcmp0 (tooltip, adw_split_button_get_dropdown_tooltip (self)))
+    return;
+
+  gtk_widget_set_tooltip_markup (self->menu_button, tooltip);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DROPDOWN_TOOLTIP]);
+}
+
 /**
  * adw_split_button_popup:
  * @self: a split button
  *
  * Pops up the menu.
- *
- * Since: 1.0
  */
 void
 adw_split_button_popup (AdwSplitButton *self)
@@ -917,8 +958,6 @@ adw_split_button_popup (AdwSplitButton *self)
  * @self: a split button
  *
  * Dismisses the menu.
- *
- * Since: 1.0
  */
 void
 adw_split_button_popdown (AdwSplitButton *self)
