@@ -16,7 +16,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -71,16 +70,12 @@
  * ## CSS nodes
  *
  * `AdwViewStack` has a single CSS node named `stack`.
- *
- * Since: 1.0
  */
 
 /**
  * AdwViewStackPage:
  *
  * An auxiliary class used by [class@ViewStack].
- *
- * Since: 1.0
  */
 
 #define OPPOSITE_ORIENTATION(_orientation) (1 - (_orientation))
@@ -117,11 +112,11 @@ enum {
   PAGE_PROP_CHILD,
   PAGE_PROP_NAME,
   PAGE_PROP_TITLE,
+  PAGE_PROP_USE_UNDERLINE,
   PAGE_PROP_ICON_NAME,
   PAGE_PROP_NEEDS_ATTENTION,
   PAGE_PROP_BADGE_NUMBER,
   PAGE_PROP_VISIBLE,
-  PAGE_PROP_USE_UNDERLINE,
   LAST_PAGE_PROP
 };
 
@@ -166,6 +161,9 @@ adw_view_stack_page_get_property (GObject      *object,
   case PAGE_PROP_TITLE:
     g_value_set_string (value, adw_view_stack_page_get_title (self));
     break;
+  case PAGE_PROP_USE_UNDERLINE:
+    g_value_set_boolean (value, adw_view_stack_page_get_use_underline (self));
+    break;
   case PAGE_PROP_ICON_NAME:
     g_value_set_string (value, adw_view_stack_page_get_icon_name (self));
     break;
@@ -178,9 +176,6 @@ adw_view_stack_page_get_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     g_value_set_boolean (value, adw_view_stack_page_get_visible (self));
     break;
-  case PAGE_PROP_USE_UNDERLINE:
-    g_value_set_boolean (value, adw_view_stack_page_get_use_underline (self));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -205,6 +200,9 @@ adw_view_stack_page_set_property (GObject      *object,
   case PAGE_PROP_TITLE:
     adw_view_stack_page_set_title (self, g_value_get_string (value));
     break;
+  case PAGE_PROP_USE_UNDERLINE:
+    adw_view_stack_page_set_use_underline (self, g_value_get_boolean (value));
+    break;
   case PAGE_PROP_ICON_NAME:
     adw_view_stack_page_set_icon_name (self, g_value_get_string (value));
     break;
@@ -217,9 +215,6 @@ adw_view_stack_page_set_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     adw_view_stack_page_set_visible (self, g_value_get_boolean (value));
     break;
-  case PAGE_PROP_USE_UNDERLINE:
-    adw_view_stack_page_set_use_underline (self, g_value_get_boolean (value));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -255,9 +250,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   /**
    * AdwViewStackPage:child: (attributes org.gtk.Property.get=adw_view_stack_page_get_child)
    *
-   * The child of the page.
-   *
-   * Since: 1.0
+   * The stack child to which the page belongs.
    */
   page_props[PAGE_PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -268,8 +261,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:name: (attributes org.gtk.Property.get=adw_view_stack_page_get_name org.gtk.Property.set=adw_view_stack_page_set_name)
    *
    * The name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NAME] =
     g_param_spec_string ("name", NULL, NULL,
@@ -280,20 +271,26 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:title: (attributes org.gtk.Property.get=adw_view_stack_page_get_title org.gtk.Property.set=adw_view_stack_page_set_title)
    *
    * The title of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
+   *
+   * Whether an embedded underline in the title indicates a mnemonic.
+   */
+  page_props[PAGE_PROP_USE_UNDERLINE] =
+    g_param_spec_boolean ("use-underline", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwViewStackPage:icon-name: (attributes org.gtk.Property.get=adw_view_stack_page_get_icon_name org.gtk.Property.set=adw_view_stack_page_set_icon_name)
    *
    * The icon name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -306,8 +303,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * Whether the page requires the user attention.
    *
    * [class@ViewSwitcher] will display it as a dot next to the page icon.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NEEDS_ATTENTION] =
     g_param_spec_boolean ("needs-attention", NULL, NULL,
@@ -317,14 +312,12 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   /**
    * AdwViewStackPage:badge-number: (attributes org.gtk.Property.get=adw_view_stack_page_get_badge_number org.gtk.Property.set=adw_view_stack_page_set_badge_number)
    *
-   * A number associated with the page.
+   * The badge number for this page.
    *
    * [class@ViewSwitcher] can display it as a badge next to the page icon. It is
    * commonly used to display a number of unread items within the page.
    *
    * It can be used together with [property@ViewStack{age}:needs-attention].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_BADGE_NUMBER] =
     g_param_spec_uint ("badge-number", NULL, NULL,
@@ -338,26 +331,12 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    *
    * This is independent from the [property@Gtk.Widget:visible] property of
    * [property@ViewStackPage:child].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_VISIBLE] =
     g_param_spec_boolean ("visible", NULL, NULL,
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
-   *
-   * Whether an embedded underline in the title indicates a mnemonic.
-   *
-   * Since: 1.0
-   */
-  page_props[PAGE_PROP_USE_UNDERLINE] =
-    g_param_spec_boolean ("use-underline", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   g_object_class_install_properties (object_class, LAST_PAGE_PROP, page_props);
 }
 
@@ -683,7 +662,8 @@ static AdwViewStackPage *
 add_internal (AdwViewStack *self,
               GtkWidget    *child,
               const char   *name,
-              const char   *title)
+              const char   *title,
+              const char   *icon_name)
 {
   AdwViewStackPage *page;
 
@@ -693,7 +673,7 @@ add_internal (AdwViewStack *self,
   page->widget = g_object_ref (child);
   page->name = g_strdup (name);
   page->title = g_strdup (title);
-  page->icon_name = NULL;
+  page->icon_name = g_strdup (icon_name);
   page->needs_attention = FALSE;
   page->last_focus = NULL;
 
@@ -897,12 +877,13 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   /**
    * AdwViewStack:hhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_hhomogeneous org.gtk.Property.set=adw_view_stack_set_hhomogeneous)
    *
-   * Whether the stack allocates the same width for all children.
+   * Whether the stack is horizontally homogeneous.
+   *
+   * If the stack is horizontally homogeneous, it allocates the same width for
+   * all children.
    *
    * If it's `FALSE`, the stack may change width when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_HHOMOGENEOUS] =
     g_param_spec_boolean ("hhomogeneous", NULL, NULL,
@@ -912,12 +893,13 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   /**
    * AdwViewStack:vhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_vhomogeneous org.gtk.Property.set=adw_view_stack_set_vhomogeneous)
    *
-   * Whether the stack allocates the same height for all children.
+   * Whether the stack is vertically homogeneous.
+   *
+   * If the stack is vertically homogeneous, it allocates the same height for
+   * all children.
    *
    * If it's `FALSE`, the stack may change height when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_VHOMOGENEOUS] =
     g_param_spec_boolean ("vhomogeneous", NULL, NULL,
@@ -928,8 +910,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * AdwViewStack:visible-child: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child org.gtk.Property.set=adw_view_stack_set_visible_child)
    *
    * The widget currently visible in the stack.
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD] =
     g_param_spec_object ("visible-child", NULL, NULL,
@@ -942,8 +922,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * The name of the widget currently visible in the stack.
    *
    * See [property@ViewStack:visible-child].
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD_NAME] =
     g_param_spec_string ("visible-child-name", NULL, NULL,
@@ -988,7 +966,7 @@ adw_view_stack_buildable_add_child (GtkBuildable *buildable,
   if (ADW_IS_VIEW_STACK_PAGE (child))
     add_page (self, ADW_VIEW_STACK_PAGE (child));
   else if (GTK_IS_WIDGET (child))
-    add_internal (self, GTK_WIDGET (child), NULL, NULL);
+    add_internal (self, GTK_WIDGET (child), NULL, NULL, NULL);
   else
     parent_buildable_iface->add_child (buildable, builder, child, type);
 }
@@ -1008,8 +986,6 @@ adw_view_stack_buildable_init (GtkBuildableIface *iface)
  * Gets the stack child to which @self belongs.
  *
  * Returns: (transfer none): the child to which @self belongs
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_page_get_child (AdwViewStackPage *self)
@@ -1019,178 +995,6 @@ adw_view_stack_page_get_child (AdwViewStackPage *self)
   return self->widget;
 }
 
-/**
- * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
- * @self: a view stack page
- *
- * Gets whether @self is visible in its `AdwViewStack`.
- *
- * This is independent from the [property@Gtk.Widget:visible]
- * property of its widget.
- *
- * Returns: whether @self is visible
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_visible (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
-
-  return self->visible;
-}
-
-/**
- * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
- * @self: a view stack page
- * @visible: whether @self is visible
- *
- * Sets whether @page is visible in its `AdwViewStack`.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_visible (AdwViewStackPage *self,
-                                 gboolean          visible)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  visible = !!visible;
-
-  if (visible == self->visible)
-    return;
-
-  self->visible = visible;
-
-  if (self->widget && gtk_widget_get_parent (self->widget))
-    update_child_visible (ADW_VIEW_STACK (gtk_widget_get_parent (self->widget)), self);
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_VISIBLE]);
-}
-
-/**
- * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
- * @self: a view stack page
- *
- * Gets whether the page is marked as “needs attention”.
- *
- * Returns: whether the page needs attention
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
-
-  return self->needs_attention;
-}
-
-/**
- * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
- * @self: a view stack page
- * @needs_attention: the new value to set
- *
- * Sets whether the page is marked as “needs attention”.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
-                                         gboolean          needs_attention)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  needs_attention = !!needs_attention;
-
-  if (needs_attention == self->needs_attention)
-    return;
-
-  self->needs_attention = needs_attention;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NEEDS_ATTENTION]);
-}
-
-/**
- * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
- * @self: a view stack page
- *
- * Gets the badge number for this page.
- *
- * Returns: the badge number for this page
- *
- * Since: 1.0
- */
-guint
-adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), 0);
-
-  return self->badge_number;
-}
-
-/**
- * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
- * @self: a view stack page
- * @badge_number: the new value to set
- *
- * Sets the badge number for this page.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
-                                      guint             badge_number)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  if (badge_number == self->badge_number)
-    return;
-
-  self->badge_number = badge_number;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_BADGE_NUMBER]);
-}
-
-/**
- * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
- * @self: a view stack page
- *
- * Gets whether underlines in the page title indicate mnemonics.
- *
- * Returns: whether underlines in the page title indicate mnemonics
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
-{
-  return self->use_underline;
-}
-
-/**
- * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
- * @self: a view stack page
- * @use_underline: the new value to set
- *
- * Sets whether underlines in the page title indicate mnemonics.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
-                                       gboolean          use_underline)
-{
-  use_underline = !!use_underline;
-
-  if (use_underline == self->use_underline)
-    return;
-
-  self->use_underline = use_underline;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_USE_UNDERLINE]);
-}
-
 /**
  * adw_view_stack_page_get_name: (attributes org.gtk.Method.get_property=name)
  * @self: a view stack page
@@ -1198,8 +1002,6 @@ adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
  * Gets the name of the page.
  *
  * Returns: (nullable): the name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_name (AdwViewStackPage *self)
@@ -1215,8 +1017,6 @@ adw_view_stack_page_get_name (AdwViewStackPage *self)
  * @name: (nullable): the page name
  *
  * Sets the name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_name (AdwViewStackPage *self,
@@ -1266,8 +1066,6 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
  * Gets the page title.
  *
  * Returns: (nullable): the page title
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_title (AdwViewStackPage *self)
@@ -1283,8 +1081,6 @@ adw_view_stack_page_get_title (AdwViewStackPage *self)
  * @title: (nullable): the page title
  *
  * Sets the page title.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_title (AdwViewStackPage *self,
@@ -1301,6 +1097,41 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_TITLE]);
 }
 
+/**
+ * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * @self: a view stack page
+ *
+ * Gets whether underlines in the page title indicate mnemonics.
+ *
+ * Returns: whether underlines in the page title indicate mnemonics
+ */
+gboolean
+adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
+{
+  return self->use_underline;
+}
+
+/**
+ * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * @self: a view stack page
+ * @use_underline: the new value to set
+ *
+ * Sets whether underlines in the page title indicate mnemonics.
+ */
+void
+adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
+                                       gboolean          use_underline)
+{
+  use_underline = !!use_underline;
+
+  if (use_underline == self->use_underline)
+    return;
+
+  self->use_underline = use_underline;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_USE_UNDERLINE]);
+}
+
 /**
  * adw_view_stack_page_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
  * @self: a view stack page
@@ -1308,8 +1139,6 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
  * Gets the icon name of the page.
  *
  * Returns: (nullable): the icon name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
@@ -1325,8 +1154,6 @@ adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
  * @icon_name: (nullable): the icon name
  *
  * Sets the icon name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
@@ -1343,14 +1170,143 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_ICON_NAME]);
 }
 
+/**
+ * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
+ * @self: a view stack page
+ *
+ * Gets whether the page requires the user attention.
+ *
+ * Returns: whether the page needs attention
+ */
+gboolean
+adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
+
+  return self->needs_attention;
+}
+
+/**
+ * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
+ * @self: a view stack page
+ * @needs_attention: the new value to set
+ *
+ * Sets whether the page requires the user attention.
+ *
+ * [class@ViewSwitcher] will display it as a dot next to the page icon.
+ */
+void
+adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
+                                         gboolean          needs_attention)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  needs_attention = !!needs_attention;
+
+  if (needs_attention == self->needs_attention)
+    return;
+
+  self->needs_attention = needs_attention;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NEEDS_ATTENTION]);
+}
+
+/**
+ * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
+ * @self: a view stack page
+ *
+ * Gets the badge number for this page.
+ *
+ * Returns: the badge number for this page
+ */
+guint
+adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), 0);
+
+  return self->badge_number;
+}
+
+/**
+ * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
+ * @self: a view stack page
+ * @badge_number: the new value to set
+ *
+ * Sets the badge number for this page.
+ *
+ * [class@ViewSwitcher] can display it as a badge next to the page icon. It is
+ * commonly used to display a number of unread items within the page.
+ *
+ * It can be used together with [property@ViewStack{age}:needs-attention].
+ */
+void
+adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
+                                      guint             badge_number)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  if (badge_number == self->badge_number)
+    return;
+
+  self->badge_number = badge_number;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_BADGE_NUMBER]);
+}
+
+/**
+ * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
+ * @self: a view stack page
+ *
+ * Gets whether @self is visible in its `AdwViewStack`.
+ *
+ * This is independent from the [property@Gtk.Widget:visible]
+ * property of its widget.
+ *
+ * Returns: whether @self is visible
+ */
+gboolean
+adw_view_stack_page_get_visible (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
+
+  return self->visible;
+}
+
+/**
+ * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
+ * @self: a view stack page
+ * @visible: whether @self is visible
+ *
+ * Sets whether @page is visible in its `AdwViewStack`.
+ *
+ * This is independent from the [property@Gtk.Widget:visible] property of
+ * [property@ViewStackPage:child].
+ */
+void
+adw_view_stack_page_set_visible (AdwViewStackPage *self,
+                                 gboolean          visible)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  visible = !!visible;
+
+  if (visible == self->visible)
+    return;
+
+  self->visible = visible;
+
+  if (self->widget && gtk_widget_get_parent (self->widget))
+    update_child_visible (ADW_VIEW_STACK (gtk_widget_get_parent (self->widget)), self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_VISIBLE]);
+}
+
 /**
  * adw_view_stack_new:
  *
  * Creates a new `AdwViewStack`.
  *
  * Returns: the newly created `AdwViewStack`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_new (void)
@@ -1366,8 +1322,6 @@ adw_view_stack_new (void)
  * Adds a child to @self.
  *
  * Returns: (transfer none): the [class@ViewStackPage] for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add (AdwViewStack   *self,
@@ -1376,7 +1330,7 @@ adw_view_stack_add (AdwViewStack   *self,
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
-  return add_internal (self, child, NULL, NULL);
+  return add_internal (self, child, NULL, NULL, NULL);
 }
 
 /**
@@ -1390,8 +1344,6 @@ adw_view_stack_add (AdwViewStack   *self,
  * The child is identified by the @name.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_named (AdwViewStack   *self,
@@ -1401,7 +1353,7 @@ adw_view_stack_add_named (AdwViewStack   *self,
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
-  return add_internal (self, child, name, NULL);
+  return add_internal (self, child, name, NULL, NULL);
 }
 
 /**
@@ -1417,8 +1369,6 @@ adw_view_stack_add_named (AdwViewStack   *self,
  * [class@ViewSwitcher] to represent @child, so it should be short.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_titled (AdwViewStack   *self,
@@ -1429,7 +1379,37 @@ adw_view_stack_add_titled (AdwViewStack   *self,
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
-  return add_internal (self, child, name, title);
+  return add_internal (self, child, name, title, NULL);
+}
+
+/**
+ * adw_view_stack_add_titled_with_icon:
+ * @self: a view stack
+ * @child: the widget to add
+ * @name: (nullable): the name for @child
+ * @title: a human-readable title for @child
+ * @icon_name: an icon name for @child
+ *
+ * Adds a child to @self.
+ *
+ * The child is identified by the @name. The @title and @icon_name will be used
+ * by [class@ViewSwitcher] to represent @child.
+ *
+ * Returns: (transfer none): the `AdwViewStackPage` for @child
+ *
+ * Since: 1.2
+ */
+AdwViewStackPage *
+adw_view_stack_add_titled_with_icon (AdwViewStack *self,
+                                     GtkWidget    *child,
+                                     const char   *name,
+                                     const char   *title,
+                                     const char   *icon_name)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
+  g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+
+  return add_internal (self, child, name, title, icon_name);
 }
 
 /**
@@ -1438,8 +1418,6 @@ adw_view_stack_add_titled (AdwViewStack   *self,
  * @child: the child to remove
  *
  * Removes a child widget from @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_remove (AdwViewStack  *self,
@@ -1473,8 +1451,6 @@ adw_view_stack_remove (AdwViewStack  *self,
  * Gets the [class@ViewStackPage] object for @child.
  *
  * Returns: (transfer none): the page object for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_get_page (AdwViewStack  *self,
@@ -1494,8 +1470,6 @@ adw_view_stack_get_page (AdwViewStack  *self,
  * Finds the child with @name in @self.
  *
  * Returns: (transfer none) (nullable): the requested child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_child_by_name (AdwViewStack *self,
@@ -1518,8 +1492,6 @@ adw_view_stack_get_child_by_name (AdwViewStack *self,
  * Gets the currently visible child of @self, .
  *
  * Returns: (transfer none) (nullable): the visible child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_visible_child (AdwViewStack *self)
@@ -1535,8 +1507,6 @@ adw_view_stack_get_visible_child (AdwViewStack *self)
  * @child: a child of @self
  *
  * Makes @child the visible child of @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_visible_child (AdwViewStack *self,
@@ -1566,8 +1536,6 @@ adw_view_stack_set_visible_child (AdwViewStack *self,
  * Returns the name of the currently visible child of @self.
  *
  * Returns: (transfer none) (nullable): the name of the visible child
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_get_visible_child_name (AdwViewStack *self)
@@ -1584,7 +1552,7 @@ adw_view_stack_get_visible_child_name (AdwViewStack *self)
  *
  * Makes the child with @name visible.
  *
- * Since: 1.0
+ * See [property@ViewStack:visible-child].
  */
 void
 adw_view_stack_set_visible_child_name (AdwViewStack *self,
@@ -1609,6 +1577,22 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
     set_visible_child (self, page);
 }
 
+/**
+ * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * @self: a view stack
+ *
+ * Gets whether @self is horizontally homogeneous.
+ *
+ * Returns: whether @self is horizontally homogeneous
+ */
+gboolean
+adw_view_stack_get_hhomogeneous (AdwViewStack *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
+
+  return self->homogeneous[GTK_ORIENTATION_HORIZONTAL];
+}
+
 /**
  * adw_view_stack_set_hhomogeneous: (attributes org.gtk.Method.set_property=hhomogeneous)
  * @self: a view stack
@@ -1616,7 +1600,11 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
  *
  * Sets @self to be horizontally homogeneous or not.
  *
- * Since: 1.0
+ * If the stack is horizontally homogeneous, it allocates the same width for
+ * all children.
+ *
+ * If it's `FALSE`, the stack may change width when a different child becomes
+ * visible.
  */
 void
 adw_view_stack_set_hhomogeneous (AdwViewStack *self,
@@ -1638,21 +1626,19 @@ adw_view_stack_set_hhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
  * @self: a view stack
  *
- * Gets whether @self is horizontally homogeneous.
+ * Gets whether @self is vertically homogeneous.
  *
- * Returns: whether @self is horizontally homogeneous
- *
- * Since: 1.0
+ * Returns: whether @self is vertically homogeneous
  */
 gboolean
-adw_view_stack_get_hhomogeneous (AdwViewStack *self)
+adw_view_stack_get_vhomogeneous (AdwViewStack *self)
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
 
-  return self->homogeneous[GTK_ORIENTATION_HORIZONTAL];
+  return self->homogeneous[GTK_ORIENTATION_VERTICAL];
 }
 
 /**
@@ -1662,7 +1648,11 @@ adw_view_stack_get_hhomogeneous (AdwViewStack *self)
  *
  * Sets @self to be vertically homogeneous or not.
  *
- * Since: 1.0
+ * If the stack is vertically homogeneous, it allocates the same height for
+ * all children.
+ *
+ * If it's `FALSE`, the stack may change height when a different child becomes
+ * visible.
  */
 void
 adw_view_stack_set_vhomogeneous (AdwViewStack *self,
@@ -1683,24 +1673,6 @@ adw_view_stack_set_vhomogeneous (AdwViewStack *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VHOMOGENEOUS]);
 }
 
-/**
- * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
- * @self: a view stack
- *
- * Gets whether @self is vertically homogeneous.
- *
- * Returns: whether @self is vertically homogeneous
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_get_vhomogeneous (AdwViewStack *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
-
-  return self->homogeneous[GTK_ORIENTATION_VERTICAL];
-}
-
 /**
  * adw_view_stack_get_pages: (attributes org.gtk.Method.get_property=pages)
  * @self: a view stack
@@ -1712,8 +1684,6 @@ adw_view_stack_get_vhomogeneous (AdwViewStack *self)
  * page.
  *
  * Returns: (transfer full): a `GtkSelectionModel` for the stack's children
- *
- * Since: 1.0
  */
 GtkSelectionModel *
 adw_view_stack_get_pages (AdwViewStack *self)
